@@ -87,7 +87,9 @@ static void _getUniquePortName(CarlaString& sname, const CarlaStringList& list)
 // Carla Engine Client
 
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
-CarlaEngineClient::ProtectedData::ProtectedData(const CarlaEngine& eng, EngineInternalGraph& eg, CarlaPlugin* const p) noexcept
+CarlaEngineClient::ProtectedData::ProtectedData(const CarlaEngine& eng,
+                                                EngineInternalGraph& eg,
+                                                const CarlaPluginPtr p) noexcept
 #else
 CarlaEngineClient::ProtectedData::ProtectedData(const CarlaEngine& eng) noexcept
 #endif
@@ -106,6 +108,14 @@ CarlaEngineClient::ProtectedData::ProtectedData(const CarlaEngine& eng) noexcept
        eventInList(),
        eventOutList() {}
 
+CarlaEngineClient::ProtectedData::~ProtectedData()
+{
+    carla_debug("CarlaEngineClient::ProtectedData::~ProtectedData()");
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+    CARLA_SAFE_ASSERT(plugin.get() == nullptr);
+#endif
+}
+
 void CarlaEngineClient::ProtectedData::addAudioPortName(const bool isInput, const char* const name)
 {
     CARLA_SAFE_ASSERT_RETURN(name != nullptr && name[0] != '\0',);
@@ -178,12 +188,20 @@ void CarlaEngineClient::activate() noexcept
     pData->active = true;
 }
 
-void CarlaEngineClient::deactivate() noexcept
+void CarlaEngineClient::deactivate(const bool willClose) noexcept
 {
-    CARLA_SAFE_ASSERT(pData->active);
-    carla_debug("CarlaEngineClient::deactivate()");
+    CARLA_SAFE_ASSERT(pData->active || willClose);
+    carla_debug("CarlaEngineClient::deactivate(%s)", bool2str(willClose));
 
     pData->active = false;
+
+    if (willClose)
+    {
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+        pData->cvSourcePorts.resetGraphAndPlugin();
+        pData->plugin.reset();
+#endif
+    }
 }
 
 bool CarlaEngineClient::isActive() const noexcept
