@@ -1002,8 +1002,7 @@ const char* getJuceApiName(const uint uindex)
 {
     initJuceDevicesIfNeeded();
 
-    const int index(static_cast<int>(uindex));
-
+    const int index = static_cast<int>(uindex);
     CARLA_SAFE_ASSERT_RETURN(index < gDeviceTypes.size(), nullptr);
 
     juce::AudioIODeviceType* const deviceType(gDeviceTypes[index]);
@@ -1016,8 +1015,7 @@ const char* const* getJuceApiDeviceNames(const uint uindex)
 {
     initJuceDevicesIfNeeded();
 
-    const int index(static_cast<int>(uindex));
-
+    const int index = static_cast<int>(uindex);
     CARLA_SAFE_ASSERT_RETURN(index < gDeviceTypes.size(), nullptr);
 
     juce::AudioIODeviceType* const deviceType(gDeviceTypes[index]);
@@ -1045,8 +1043,7 @@ const EngineDriverDeviceInfo* getJuceDeviceInfo(const uint uindex, const char* c
 {
     initJuceDevicesIfNeeded();
 
-    const int index(static_cast<int>(uindex));
-
+    const int index = static_cast<int>(uindex);
     CARLA_SAFE_ASSERT_RETURN(index < gDeviceTypes.size(), nullptr);
 
     juce::AudioIODeviceType* const deviceType(gDeviceTypes[index]);
@@ -1083,7 +1080,7 @@ const EngineDriverDeviceInfo* getJuceDeviceInfo(const uint uindex, const char* c
         devInfo.hints |= ENGINE_DRIVER_DEVICE_HAS_CONTROL_PANEL;
 
     juce::Array<int> juceBufferSizes = device->getAvailableBufferSizes();
-    if (int bufferSizesCount = juceBufferSizes.size())
+    if (const int bufferSizesCount = juceBufferSizes.size())
     {
         uint32_t* const bufferSizes(new uint32_t[bufferSizesCount+1]);
 
@@ -1099,7 +1096,7 @@ const EngineDriverDeviceInfo* getJuceDeviceInfo(const uint uindex, const char* c
     }
 
     juce::Array<double> juceSampleRates = device->getAvailableSampleRates();
-    if (int sampleRatesCount = juceSampleRates.size())
+    if (const int sampleRatesCount = juceSampleRates.size())
     {
         double* const sampleRates(new double[sampleRatesCount+1]);
 
@@ -1119,8 +1116,7 @@ const EngineDriverDeviceInfo* getJuceDeviceInfo(const uint uindex, const char* c
 
 bool showJuceDeviceControlPanel(const uint uindex, const char* const deviceName)
 {
-    const int index(static_cast<int>(uindex));
-
+    const int index = static_cast<int>(uindex);
     CARLA_SAFE_ASSERT_RETURN(index < gDeviceTypes.size(), false);
 
     juce::AudioIODeviceType* const deviceType(gDeviceTypes[index]);
