@@ -1,6 +1,6 @@
 /*
  * Carla Juce Plugin
- * Copyright (C) 2013-2021 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2013-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,26 +27,18 @@
 #include "CarlaVst2Utils.hpp"
 #include "CarlaVst3Utils.hpp"
 
-#if defined(__clang__)
-# pragma clang diagnostic push
-# pragma clang diagnostic ignored "-Wfloat-equal"
-# pragma clang diagnostic ignored "-Wimplicit-int-float-conversion"
-#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
-# pragma GCC diagnostic push
-# pragma GCC diagnostic ignored "-Wconversion"
-# pragma GCC diagnostic ignored "-Wdouble-promotion"
-# pragma GCC diagnostic ignored "-Weffc++"
-# pragma GCC diagnostic ignored "-Wfloat-equal"
-#endif
-
 #define JUCE_GUI_BASICS_INCLUDE_XHEADERS 1
-#include "AppConfig.h"
+
+#include "carla_juce/carla_juce.h"
+
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdouble-promotion"
+#pragma GCC diagnostic ignored "-Wduplicated-branches"
+#pragma GCC diagnostic ignored "-Weffc++"
+#pragma GCC diagnostic ignored "-Wfloat-equal"
 #include "juce_audio_processors/juce_audio_processors.h"
 #include "juce_gui_basics/juce_gui_basics.h"
-
-#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
-# pragma GCC diagnostic pop
-#endif
+#pragma GCC diagnostic pop
 
 #include "JucePluginWindow.hpp"
 
@@ -97,6 +89,44 @@ static void findMaxTotalChannels(juce::AudioProcessor* const filter,
     }
 }
 
+// -------------------------------------------------------------------------------------------------------------------
+// ExtensionsVisitor for various formats
+
+static void* getPlatformSpecificDataForAll(const std::unique_ptr<juce::AudioPluginInstance>& instance)
+{
+    struct ExtensionsVisitorForVST2 : public juce::ExtensionsVisitor
+    {
+        void* ptr = nullptr;
+        void visitVST3Client     (const VST3Client& c)      override { ptr = c.getIComponentPtr(); }
+        void visitVSTClient      (const VSTClient& c)       override { ptr = c.getAEffectPtr(); }
+        void visitAudioUnitClient(const AudioUnitClient& c) override { ptr = c.getAudioUnitHandle(); }
+    } visitor;
+    instance->getExtensions(visitor);
+    return visitor.ptr;
+}
+
+static AEffect* getPlatformSpecificDataForVST2(const std::unique_ptr<juce::AudioPluginInstance>& instance)
+{
+    struct ExtensionsVisitorForVST2 : public juce::ExtensionsVisitor
+    {
+        AEffect* aeffect = nullptr;
+        void visitVSTClient(const VSTClient& c) override { aeffect = c.getAEffectPtr(); }
+    } visitor;
+    instance->getExtensions(visitor);
+    return visitor.aeffect;
+}
+
+static v3_component** getPlatformSpecificDataForVST3(const std::unique_ptr<juce::AudioPluginInstance>& instance)
+{
+    struct ExtensionsVisitorForVST3 : public juce::ExtensionsVisitor
+    {
+        v3_component** component = nullptr;
+        void visitVST3Client(const VST3Client& c) override { component = (v3_component**)c.getIComponentPtr(); }
+    } visitor;
+    instance->getExtensions(visitor);
+    return visitor.component;
+}
+
 // -------------------------------------------------------------------------------------------------------------------
 
 class CarlaPluginJuce : public CarlaPlugin,
@@ -114,13 +144,13 @@ public:
           fPosInfo(),
           fChunk(),
           fFormatName(),
-          fWindow()
+          fWindow(),
+          fNeedsUpdate(false)
     {
         carla_debug("CarlaPluginJuce::CarlaPluginJuce(%p, %i)", engine, id);
 
         fMidiBuffer.ensureSize(2048);
         fMidiBuffer.clear();
-        fPosInfo.resetToDefault();
     }
 
     ~CarlaPluginJuce() override
@@ -167,7 +197,17 @@ public:
 
     int64_t getUniqueId() const noexcept override
     {
-        return fDesc.uid;
+        return fDesc.uniqueId;
+    }
+
+    uint32_t getLatencyInFrames() const noexcept override
+    {
+        CARLA_SAFE_ASSERT_RETURN(fInstance != nullptr, 0);
+
+        const int latency = fInstance->getLatencySamples();
+        CARLA_SAFE_ASSERT_RETURN(latency >= 0, 0);
+
+        return static_cast<uint32_t>(latency);
     }
 
     // -------------------------------------------------------------------
@@ -310,7 +350,7 @@ public:
         if (fDesc.pluginFormatName != "VST" && fDesc.pluginFormatName != "VST2")
             return false;
 
-        AEffect* const effect = (AEffect*)fInstance->getPlatformSpecificData();
+        AEffect* const effect = getPlatformSpecificDataForVST2(fInstance);
 
         if (effect == nullptr)
             return false;
@@ -368,7 +408,7 @@ public:
         CarlaPlugin::setParameterValue(parameterId, fixedValue, sendGui, sendOsc, sendCallback);
     }
 
-    void setParameterValueRT(const uint32_t parameterId, const float value, const bool sendCallbackLater) noexcept override
+    void setParameterValueRT(const uint32_t parameterId, const float value, const uint32_t frameOffset, const bool sendCallbackLater) noexcept override
     {
         CARLA_SAFE_ASSERT_RETURN(parameterId < pData->param.count,);
         CARLA_SAFE_ASSERT_RETURN(fInstance != nullptr,);
@@ -382,7 +422,7 @@ public:
             parameter->setValue(value);
         } CARLA_SAFE_EXCEPTION("setValue(RT)");
 
-        CarlaPlugin::setParameterValueRT(parameterId, fixedValue, sendCallbackLater);
+        CarlaPlugin::setParameterValueRT(parameterId, fixedValue, frameOffset, sendCallbackLater);
     }
 
     void setChunkData(const void* const data, const std::size_t dataSize) override
@@ -497,12 +537,15 @@ public:
                     }
 
                     AEffect* const vst2effect = fDesc.pluginFormatName == "VST" || fDesc.pluginFormatName == "VST2"
-                                              ? (AEffect*)fInstance->getPlatformSpecificData()
+                                              ? getPlatformSpecificDataForVST2(fInstance)
                                               : nullptr;
 
+                    /* TODO update to juce7 APIs
                     v3_plugin_view** const vst3view = fDesc.pluginFormatName == "VST3"
                                                     ? (v3_plugin_view**)editor->getPlatformSpecificData()
                                                     : nullptr;
+                    */
+                    v3_plugin_view** const vst3view = nullptr;
 
                     fWindow = new JucePluginWindow(opts.frontendWinId, opts.pluginsAreStandalone,
                                                    vst2effect, vst3view);
@@ -679,7 +722,7 @@ public:
 
             if (isVST2)
             {
-                AEffect* const effect = (AEffect*)fInstance->getPlatformSpecificData();
+                AEffect* const effect = getPlatformSpecificDataForVST2(fInstance);
 
                 VstParameterProperties prop;
                 carla_zeroStruct(prop);
@@ -723,8 +766,9 @@ public:
             }
             else if (isVST3)
             {
-                v3_component** const component = (v3_component**)fInstance->getPlatformSpecificData();
+                v3_component** const component = getPlatformSpecificDataForVST3(fInstance);
 
+                /* FIXME query edit controller the long way too */
                 v3_edit_controller** controller = nullptr;
                 v3_cpp_obj_query_interface(component, v3_edit_controller_iid, &controller);
 
@@ -871,10 +915,8 @@ public:
         if (mOuts > 0)
             pData->extraHints |= PLUGIN_EXTRA_HINT_HAS_MIDI_OUT;
 
-        fInstance->setPlayConfigDetails(static_cast<int>(aIns),
-                                        static_cast<int>(aOuts),
-                                        pData->engine->getSampleRate(),
-                                        static_cast<int>(pData->engine->getBufferSize()));
+        fInstance->setRateAndBufferSizeDetails(pData->engine->getSampleRate(),
+                                               static_cast<int>(pData->engine->getBufferSize()));
 
         bufferSizeChanged(pData->engine->getBufferSize());
         reloadPrograms(true);
@@ -1082,7 +1124,7 @@ public:
 
                             ctrlEvent.handled = true;
                             value = pData->param.getFinalUnnormalizedValue(k, ctrlEvent.normalizedValue);
-                            setParameterValueRT(k, value, true);
+                            setParameterValueRT(k, value, event.time, true);
                             continue;
                         }
 
@@ -1143,7 +1185,7 @@ public:
 
                             ctrlEvent.handled = true;
                             value = pData->param.getFinalUnnormalizedValue(k, ctrlEvent.normalizedValue);
-                            setParameterValueRT(k, value, true);
+                            setParameterValueRT(k, value, event.time, true);
                         }
 
                         if ((pData->options & PLUGIN_OPTION_SEND_CONTROL_CHANGES) != 0 && ctrlEvent.param < MAX_MIDI_VALUE)
@@ -1151,7 +1193,7 @@ public:
                             uint8_t midiData[3];
                             midiData[0] = uint8_t(MIDI_STATUS_CONTROL_CHANGE | (event.channel & MIDI_CHANNEL_BIT));
                             midiData[1] = uint8_t(ctrlEvent.param);
-                            midiData[2] = uint8_t(ctrlEvent.normalizedValue*127.0f);
+                            midiData[2] = uint8_t(ctrlEvent.normalizedValue*127.0f + 0.5f);
 
                             fMidiBuffer.addEvent(midiData, 3, static_cast<int>(event.time));
                         }
@@ -1173,7 +1215,7 @@ public:
                             fMidiBuffer.addEvent(midiData, 3, static_cast<int>(event.time));
 
                             midiData[1] = MIDI_CONTROL_BANK_SELECT__LSB;
-                            midiData[2] = uint8_t(ctrlEvent.normalizedValue*127.0f);
+                            midiData[2] = uint8_t(ctrlEvent.normalizedValue*127.0f + 0.5f);
                             fMidiBuffer.addEvent(midiData, 3, static_cast<int>(event.time));
                         }
                         break;
@@ -1188,9 +1230,9 @@ public:
                         }
                         else if ((pData->options & PLUGIN_OPTION_SEND_PROGRAM_CHANGES) != 0)
                         {
-                            uint8_t midiData[3];
+                            uint8_t midiData[2];
                             midiData[0] = uint8_t(MIDI_STATUS_PROGRAM_CHANGE | (event.channel & MIDI_CHANNEL_BIT));
-                            midiData[1] = uint8_t(ctrlEvent.normalizedValue*127.0f);
+                            midiData[1] = uint8_t(ctrlEvent.normalizedValue*127.0f + 0.5f);
                             fMidiBuffer.addEvent(midiData, 2, static_cast<int>(event.time));
                         }
                         break;
@@ -1280,7 +1322,9 @@ public:
 
         const EngineTimeInfo& timeInfo(pData->engine->getTimeInfo());
 
-        fPosInfo.isPlaying = timeInfo.playing;
+        fPosInfo.setIsPlaying(timeInfo.playing);
+        fPosInfo.setTimeInSamples(static_cast<int64_t>(timeInfo.frame));
+        fPosInfo.setTimeInSeconds(static_cast<double>(timeInfo.frame)/pData->engine->getSampleRate());
 
         if (timeInfo.bbt.valid)
         {
@@ -1291,16 +1335,14 @@ public:
             const double ppqBeat = static_cast<double>(timeInfo.bbt.beat - 1);
             const double ppqTick = timeInfo.bbt.tick / timeInfo.bbt.ticksPerBeat;
 
-            fPosInfo.bpm = timeInfo.bbt.beatsPerMinute;
+            fPosInfo.setBarCount(timeInfo.bbt.bar);
+            fPosInfo.setBpm(timeInfo.bbt.beatsPerMinute);
 
-            fPosInfo.timeSigNumerator   = static_cast<int>(timeInfo.bbt.beatsPerBar);
-            fPosInfo.timeSigDenominator = static_cast<int>(timeInfo.bbt.beatType);
+            fPosInfo.setTimeSignature(TimeSignature{static_cast<int>(timeInfo.bbt.beatsPerBar+0.5f),
+                                                    static_cast<int>(timeInfo.bbt.beatType+0.5f)});
 
-            fPosInfo.timeInSamples = static_cast<int64_t>(timeInfo.frame);
-            fPosInfo.timeInSeconds = static_cast<double>(fPosInfo.timeInSamples)/pData->engine->getSampleRate();
-
-            fPosInfo.ppqPosition = ppqBar + ppqBeat + ppqTick;
-            fPosInfo.ppqPositionOfLastBarStart = ppqBar;
+            fPosInfo.setPpqPosition(ppqBar + ppqBeat + ppqTick);
+            fPosInfo.setPpqPositionOfLastBarStart(ppqBar);
         }
 
         // --------------------------------------------------------------------------------------------------------
@@ -1488,6 +1530,20 @@ public:
         }
     }
 
+    // -------------------------------------------------------------------
+    // Misc
+
+    void idle() override
+    {
+        if (fNeedsUpdate)
+        {
+            fNeedsUpdate = false;
+            pData->engine->callback(true, true, ENGINE_CALLBACK_UPDATE, pData->id, 0, 0, 0, 0.0f, nullptr);
+        }
+
+        CarlaPlugin::idle();
+    }
+
     // -------------------------------------------------------------------
     // Plugin buffers
 
@@ -1502,7 +1558,7 @@ public:
 
     void* getNativeHandle() const noexcept override
     {
-        return (fInstance != nullptr) ? fInstance->getPlatformSpecificData() : nullptr;
+        return (fInstance != nullptr) ? getPlatformSpecificDataForAll(fInstance) : nullptr;
     }
 
     // -------------------------------------------------------------------
@@ -1518,9 +1574,10 @@ protected:
         CarlaPlugin::setParameterValue(static_cast<uint32_t>(index), fixedValue, false, true, true);
     }
 
-    void audioProcessorChanged(juce::AudioProcessor*) override
+    void audioProcessorChanged(juce::AudioProcessor*, const ChangeDetails& details) override
     {
-        pData->engine->callback(true, true, ENGINE_CALLBACK_UPDATE, pData->id, 0, 0, 0, 0.0f, nullptr);
+        if (details.parameterInfoChanged || details.programChanged)
+            fNeedsUpdate = true;
     }
 
     void audioProcessorParameterChangeGestureBegin(juce::AudioProcessor*, int index) override
@@ -1537,10 +1594,9 @@ protected:
         pData->engine->touchPluginParameter(pData->id, static_cast<uint32_t>(index), false);
     }
 
-    bool getCurrentPosition(CurrentPositionInfo& result) override
+    juce::Optional<juce::AudioPlayHead::PositionInfo> getPosition() const override
     {
-        carla_copyStruct(result, fPosInfo);
-        return true;
+        return fPosInfo;
     }
 
     // -------------------------------------------------------------------
@@ -1637,7 +1693,9 @@ public:
             juce::KnownPluginList plist;
 
             {
+               #if !(defined(CARLA_OS_WASM) || defined(CARLA_OS_WIN))
                 const ScopedAbortCatcher sac;
+               #endif
 
                 for (int i = 0; i < fFormatManager.getNumFormats(); ++i)
                 {
@@ -1650,12 +1708,14 @@ public:
                         plist.scanAndAddFile(fileOrIdentifier, true, pluginDescriptions, *apformat);
                     } CARLA_SAFE_EXCEPTION_CONTINUE("scanAndAddFile")
 
+                   #if !(defined(CARLA_OS_WASM) || defined(CARLA_OS_WIN))
                     if (sac.wasTriggered())
                     {
                         carla_stderr("WARNING: Caught exception while scanning file, will not load this plugin");
                         pluginDescriptions.clearQuick(false);
                         break;
                     }
+                   #endif
                 }
             }
 
@@ -1669,12 +1729,14 @@ public:
         }
 
         if (uniqueId != 0)
-            fDesc.uid = static_cast<int>(uniqueId);
+            fDesc.uniqueId = static_cast<int>(uniqueId);
 
         juce::String error;
 
         {
+           #if !(defined(CARLA_OS_WASM) || defined(CARLA_OS_WIN))
             const ScopedAbortCatcher sac;
+           #endif
 
             try {
                 fInstance = fFormatManager.createPluginInstance(fDesc,
@@ -1683,11 +1745,13 @@ public:
                                                                 error);
             } CARLA_SAFE_EXCEPTION("createPluginInstance")
 
+           #if !(defined(CARLA_OS_WASM) || defined(CARLA_OS_WIN))
             if (sac.wasTriggered())
             {
                 fInstance = nullptr;
                 carla_stderr("WARNING: Caught exception while instantiating, will not load this plugin");
             }
+           #endif
         }
 
         if (fInstance == nullptr)
@@ -1763,12 +1827,14 @@ private:
 
     juce::AudioSampleBuffer fAudioBuffer;
     juce::MidiBuffer        fMidiBuffer;
-    CurrentPositionInfo     fPosInfo;
+    PositionInfo            fPosInfo;
     juce::MemoryBlock       fChunk;
     juce::String            fFormatName;
 
     CarlaScopedPointer<JucePluginWindow> fWindow;
 
+    bool fNeedsUpdate;
+
     bool isJuceSaveFormat(const void* const data, const std::size_t dataSize)
     {
         if (fFormatName != "VST2")
