@@ -132,7 +132,6 @@ protected:
 
 CarlaScopedPointer<DiscoveryPipe> gPipe;
 
-
 // --------------------------------------------------------------------------------------------------------------------
 // Don't print ELF/EXE related errors since discovery can find multi-architecture binaries
 
@@ -151,6 +150,16 @@ static void print_lib_error(const char* const filename)
     }
 }
 
+#ifdef CARLA_OS_WIN
+// --------------------------------------------------------------------------------------------------------------------
+// Do not show error message box on Windows
+
+static LONG WINAPI winExceptionFilter(_EXCEPTION_POINTERS*)
+{
+    return EXCEPTION_EXECUTE_HANDLER;
+}
+#endif
+
 // --------------------------------------------------------------------------------------------------------------------
 // Plugin Checks
 
@@ -2305,13 +2314,19 @@ int main(int argc, const char* argv[])
 #endif
 
 #ifdef CARLA_OS_WIN
+    // init win32 stuff that plugins might use
     OleInitialize(nullptr);
     CoInitializeEx(nullptr, COINIT_APARTMENTTHREADED);
+
 # ifndef __WINPTHREADS_VERSION
     // (non-portable) initialization of statically linked pthread library
     pthread_win32_process_attach_np();
     pthread_win32_thread_attach_np();
 # endif
+
+    // do not show error message box on Windows
+    SetErrorMode(SEM_NOGPFAULTERRORBOX);
+    SetUnhandledExceptionFilter(winExceptionFilter);
 #endif
 
     // ---------------------------------------------------------------------------------------------------------------
