@@ -21,7 +21,7 @@
 
 from math import floor
 
-from PyQt5.QtCore import qCritical, Qt, QLineF, QPointF, QRectF, QTimer
+from PyQt5.QtCore import qCritical, QT_VERSION, Qt, QLineF, QPointF, QRectF, QTimer
 from PyQt5.QtGui import QCursor, QFont, QFontMetrics, QPainter, QPainterPath, QPen, QPolygonF
 from PyQt5.QtWidgets import QGraphicsItem, QMenu
 
@@ -120,7 +120,16 @@ class CanvasPort(QGraphicsItem):
         self.update()
 
     def setPortName(self, port_name):
-        if QFontMetrics(self.m_port_font).width(port_name) < QFontMetrics(self.m_port_font).width(self.m_port_name):
+        metrics = QFontMetrics(self.m_port_font)
+
+        if QT_VERSION >= 0x50b00:
+            width1 = metrics.horizontalAdvance(port_name)
+            width2 = metrics.horizontalAdvance(self.m_port_name)
+        else:
+            width1 = metrics.width(port_name)
+            width2 = metrics.width(self.m_port_name)
+
+        if width1 < width2:
             QTimer.singleShot(0, canvas.scene.update)
 
         self.m_port_name = port_name
@@ -147,6 +156,10 @@ class CanvasPort(QGraphicsItem):
         QGraphicsItem.hoverLeaveEvent(self, event)
 
     def mousePressEvent(self, event):
+        if event.button() == Qt.MiddleButton or event.source() == Qt.MouseEventSynthesizedByApplication:
+            event.ignore()
+            return
+
         if self.m_mouse_down:
             self.handleMouseRelease()
         self.m_hover_item = None
@@ -269,7 +282,8 @@ class CanvasPort(QGraphicsItem):
         self.m_cursor_moving = False
 
     def mouseReleaseEvent(self, event):
-        self.handleMouseRelease()
+        if event.button() == Qt.LeftButton:
+            self.handleMouseRelease()
         QGraphicsItem.mouseReleaseEvent(self, event)
 
     def contextMenuEvent(self, event):
@@ -371,6 +385,7 @@ class CanvasPort(QGraphicsItem):
             conn_pen = QPen(theme.port_parameter_pen_sel)
         else:
             qCritical("PatchCanvas::CanvasPort.paint() - invalid port type '%s'" % port_type2str(self.m_port_type))
+            painter.restore()
             return
 
         # To prevent quality worsening
@@ -407,6 +422,7 @@ class CanvasPort(QGraphicsItem):
                 poly_locx[4] = lineHinting
             else:
                 qCritical("PatchCanvas::CanvasPort.paint() - invalid theme port mode '%s'" % canvas.theme.port_mode)
+                painter.restore()
                 return
 
         elif self.m_port_mode == PORT_MODE_OUTPUT:
@@ -426,10 +442,12 @@ class CanvasPort(QGraphicsItem):
                 poly_locx[4] = self.m_port_width + 12 - lineHinting
             else:
                 qCritical("PatchCanvas::CanvasPort.paint() - invalid theme port mode '%s'" % canvas.theme.port_mode)
+                painter.restore()
                 return
 
         else:
             qCritical("PatchCanvas::CanvasPort.paint() - invalid port mode '%s'" % port_mode2str(self.m_port_mode))
+            painter.restore()
             return
 
         polygon = QPolygonF()
