@@ -8,7 +8,7 @@
 //
 //-----------------------------------------------------------------------------
 // LICENSE
-// (c) 2019, Steinberg Media Technologies GmbH, All Rights Reserved
+// (c) 2021, Steinberg Media Technologies GmbH, All Rights Reserved
 //-----------------------------------------------------------------------------
 // Redistribution and use in source and binary forms, with or without modification,
 // are permitted provided that the following conditions are met:
@@ -42,7 +42,7 @@ namespace Steinberg {
 //  CPluginView implementation
 //------------------------------------------------------------------------
 CPluginView::CPluginView (const ViewRect* _rect)
-: rect (0, 0, 0, 0), systemWindow (nullptr), plugFrame (nullptr)
+: rect (0, 0, 0, 0)
 {
 	if (_rect)
 		rect = *_rect;
