@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -37,8 +37,6 @@ static void updateOnMessageThread (AsyncUpdater& updater)
 template <typename FloatType>
 struct GraphRenderSequence
 {
-    GraphRenderSequence() {}
-
     struct Context
     {
         FloatType** audioBuffers;
@@ -64,6 +62,8 @@ struct GraphRenderSequence
                 midiChunk.clear();
                 midiChunk.addEvents (midiMessages, chunkStartSample, chunkSize, -chunkStartSample);
 
+                // Splitting up the buffer like this will cause the play head and host time to be
+                // invalid for all but the first chunk...
                 perform (audioChunk, midiChunk, audioPlayHead);
 
                 chunkStartSample += maxSamples;
@@ -196,18 +196,19 @@ private:
     OwnedArray<RenderingOp> renderOps;
 
     //==============================================================================
-    template <typename LambdaType>
+    template <typename LambdaType,
+              std::enable_if_t<std::is_rvalue_reference<LambdaType&&>::value, int> = 0>
     void createOp (LambdaType&& fn)
     {
         struct LambdaOp  : public RenderingOp
         {
-            LambdaOp (LambdaType&& f) : function (std::move (f)) {}
+            LambdaOp (LambdaType&& f) : function (std::forward<LambdaType> (f)) {}
             void perform (const Context& c) override    { function (c); }
 
             LambdaType function;
         };
 
-        renderOps.add (new LambdaOp (std::move (fn)));
+        renderOps.add (new LambdaOp (std::forward<LambdaType> (fn)));
     }
 
     //==============================================================================
@@ -267,7 +268,18 @@ private:
             for (int i = 0; i < totalChans; ++i)
                 audioChannels[i] = c.audioBuffers[audioChannelsToUse.getUnchecked (i)];
 
-            AudioBuffer<FloatType> buffer (audioChannels, totalChans, c.numSamples);
+            auto numAudioChannels = [this]
+            {
+                if (const auto* proc = node->getProcessor())
+                    if (proc->getTotalNumInputChannels() == 0 && proc->getTotalNumOutputChannels() == 0)
+                        return 0;
+
+                return totalChans;
+            }();
+
+            AudioBuffer<FloatType> buffer (audioChannels, numAudioChannels, c.numSamples);
+
+            const ScopedLock lock (processor.getCallbackLock());
 
             if (processor.isSuspended())
                 buffer.clear();
@@ -337,10 +349,8 @@ template <typename RenderSequence>
 struct RenderSequenceBuilder
 {
     RenderSequenceBuilder (AudioProcessorGraph& g, RenderSequence& s)
-        : graph (g), sequence (s)
+        : graph (g), sequence (s), orderedNodes (createOrderedNodeList (graph))
     {
-        createOrderedNodeList();
-
         audioBuffers.add (AssignedBuffer::createReadOnlyEmpty()); // first buffer is read-only zeros
         midiBuffers .add (AssignedBuffer::createReadOnlyEmpty());
 
@@ -358,12 +368,13 @@ struct RenderSequenceBuilder
     }
 
     //==============================================================================
+    using Node = AudioProcessorGraph::Node;
     using NodeID = AudioProcessorGraph::NodeID;
 
     AudioProcessorGraph& graph;
     RenderSequence& sequence;
 
-    Array<AudioProcessorGraph::Node*> orderedNodes;
+    const Array<Node*> orderedNodes;
 
     struct AssignedBuffer
     {
@@ -415,21 +426,58 @@ struct RenderSequenceBuilder
     }
 
     //==============================================================================
-    void createOrderedNodeList()
+    static void getAllParentsOfNode (const Node* child,
+                                     std::unordered_set<Node*>& parents,
+                                     const std::unordered_map<Node*, std::unordered_set<Node*>>& otherParents)
     {
-        for (auto* node : graph.getNodes())
+        for (auto&& i : child->inputs)
         {
-            int j = 0;
+            auto* parentNode = i.otherNode;
 
-            for (; j < orderedNodes.size(); ++j)
-                if (graph.isAnInputTo (*node, *orderedNodes.getUnchecked(j)))
-                  break;
+            if (parentNode == child)
+                continue;
 
-            orderedNodes.insert (j, node);
+            if (parents.insert (parentNode).second)
+            {
+                auto parentParents = otherParents.find (parentNode);
+
+                if (parentParents != otherParents.end())
+                {
+                    parents.insert (parentParents->second.begin(), parentParents->second.end());
+                    continue;
+                }
+
+                getAllParentsOfNode (i.otherNode, parents, otherParents);
+            }
         }
     }
 
-    int findBufferForInputAudioChannel (AudioProcessorGraph::Node& node, const int inputChan,
+    static auto createOrderedNodeList (const AudioProcessorGraph& graph)
+    {
+        Array<Node*> result;
+
+        std::unordered_map<Node*, std::unordered_set<Node*>> nodeParents;
+
+        for (auto* node : graph.getNodes())
+        {
+            int insertionIndex = 0;
+
+            for (; insertionIndex < result.size(); ++insertionIndex)
+            {
+                auto& parents = nodeParents[result.getUnchecked (insertionIndex)];
+
+                if (parents.find (node) != parents.end())
+                    break;
+            }
+
+            result.insert (insertionIndex, node);
+            getAllParentsOfNode (node, nodeParents[node], nodeParents);
+        }
+
+        return result;
+    }
+
+    int findBufferForInputAudioChannel (Node& node, const int inputChan,
                                         const int ourRenderingIndex, const int maxLatency)
     {
         auto& processor = *node.getProcessor();
@@ -561,7 +609,7 @@ struct RenderSequenceBuilder
         return bufIndex;
     }
 
-    int findBufferForInputMidiChannel (AudioProcessorGraph::Node& node, int ourRenderingIndex)
+    int findBufferForInputMidiChannel (Node& node, int ourRenderingIndex)
     {
         auto& processor = *node.getProcessor();
         auto sources = getSourcesForChannel (node, AudioProcessorGraph::midiChannelIndex);
@@ -652,7 +700,7 @@ struct RenderSequenceBuilder
         return midiBufferToUse;
     }
 
-    void createRenderingOpsForNode (AudioProcessorGraph::Node& node, const int ourRenderingIndex)
+    void createRenderingOpsForNode (Node& node, const int ourRenderingIndex)
     {
         auto& processor = *node.getProcessor();
         auto numIns  = processor.getTotalNumInputChannels();
@@ -697,7 +745,7 @@ struct RenderSequenceBuilder
     }
 
     //==============================================================================
-    Array<AudioProcessorGraph::NodeAndChannel> getSourcesForChannel (AudioProcessorGraph::Node& node, int inputChannelIndex)
+    Array<AudioProcessorGraph::NodeAndChannel> getSourcesForChannel (Node& node, int inputChannelIndex)
     {
         Array<AudioProcessorGraph::NodeAndChannel> results;
         AudioProcessorGraph::NodeAndChannel nc { node.nodeID, inputChannelIndex };
@@ -1265,6 +1313,22 @@ void AudioProcessorGraph::prepareToPlay (double sampleRate, int estimatedSamples
     {
         const ScopedLock sl (getCallbackLock());
         setRateAndBufferSizeDetails (sampleRate, estimatedSamplesPerBlock);
+
+        const auto newPrepareSettings = [&]
+        {
+            PrepareSettings settings;
+            settings.precision  = getProcessingPrecision();
+            settings.sampleRate = sampleRate;
+            settings.blockSize  = estimatedSamplesPerBlock;
+            settings.valid      = true;
+            return settings;
+        }();
+
+        if (prepareSettings != newPrepareSettings)
+        {
+            unprepare();
+            prepareSettings = newPrepareSettings;
+        }
     }
 
     clearRenderingSequence();
@@ -1277,16 +1341,23 @@ bool AudioProcessorGraph::supportsDoublePrecisionProcessing() const
     return true;
 }
 
+void AudioProcessorGraph::unprepare()
+{
+    prepareSettings.valid = false;
+
+    isPrepared = 0;
+
+    for (auto* n : nodes)
+        n->unprepare();
+}
+
 void AudioProcessorGraph::releaseResources()
 {
     const ScopedLock sl (getCallbackLock());
 
     cancelPendingUpdate();
 
-    isPrepared = 0;
-
-    for (auto* n : nodes)
-        n->unprepare();
+    unprepare();
 
     if (renderSequenceFloat != nullptr)
         renderSequenceFloat->releaseBuffers();
@@ -1316,7 +1387,7 @@ void AudioProcessorGraph::setNonRealtime (bool isProcessingNonRealtime) noexcept
 double AudioProcessorGraph::getTailLengthSeconds() const            { return 0; }
 bool AudioProcessorGraph::acceptsMidi() const                       { return true; }
 bool AudioProcessorGraph::producesMidi() const                      { return true; }
-void AudioProcessorGraph::getStateInformation (juce::MemoryBlock&)  {}
+void AudioProcessorGraph::getStateInformation (MemoryBlock&)        {}
 void AudioProcessorGraph::setStateInformation (const void*, int)    {}
 
 template <typename FloatType, typename SequenceType>
@@ -1395,13 +1466,14 @@ const String AudioProcessorGraph::AudioGraphIOProcessor::getName() const
 void AudioProcessorGraph::AudioGraphIOProcessor::fillInPluginDescription (PluginDescription& d) const
 {
     d.name = getName();
-    d.uid = d.name.hashCode();
     d.category = "I/O devices";
     d.pluginFormatName = "Internal";
     d.manufacturerName = "JUCE";
     d.version = "1.0";
     d.isInstrument = false;
 
+    d.deprecatedUid = d.uniqueId = d.name.hashCode();
+
     d.numInputChannels = getTotalNumInputChannels();
 
     if (type == audioOutputNode && graph != nullptr)
@@ -1508,7 +1580,7 @@ void AudioProcessorGraph::AudioGraphIOProcessor::setCurrentProgram (int)
 const String AudioProcessorGraph::AudioGraphIOProcessor::getProgramName (int)       { return {}; }
 void AudioProcessorGraph::AudioGraphIOProcessor::changeProgramName (int, const String&) {}
 
-void AudioProcessorGraph::AudioGraphIOProcessor::getStateInformation (juce::MemoryBlock&) {}
+void AudioProcessorGraph::AudioGraphIOProcessor::getStateInformation (MemoryBlock&)     {}
 void AudioProcessorGraph::AudioGraphIOProcessor::setStateInformation (const void*, int) {}
 
 void AudioProcessorGraph::AudioGraphIOProcessor::setParentGraph (AudioProcessorGraph* const newGraph)
