@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -36,9 +36,10 @@
 
 #include "juce_core.h"
 
-#include <locale>
 #include <cctype>
 #include <cstdarg>
+#include <locale>
+#include <thread>
 
 #if ! JUCE_ANDROID
  #include <sys/timeb.h>
@@ -63,7 +64,7 @@
  #endif
 
 #else
- #if JUCE_LINUX || JUCE_ANDROID
+ #if JUCE_LINUX || JUCE_BSD || JUCE_ANDROID
   #include <sys/types.h>
   #include <sys/socket.h>
   #include <sys/errno.h>
@@ -71,7 +72,17 @@
   #include <netinet/in.h>
  #endif
 
- #if JUCE_LINUX
+ #if JUCE_WASM
+  #include <stdio.h>
+  #include <sys/types.h>
+  #include <sys/socket.h>
+  #include <errno.h>
+  #include <unistd.h>
+  #include <netinet/in.h>
+  #include <sys/stat.h>
+ #endif
+
+ #if JUCE_LINUX || JUCE_BSD
   #include <stdio.h>
   #include <langinfo.h>
   #include <ifaddrs.h>
@@ -91,7 +102,7 @@
  #include <net/if.h>
  #include <sys/ioctl.h>
 
- #if ! JUCE_ANDROID
+ #if ! (JUCE_ANDROID || JUCE_WASM)
   #include <execinfo.h>
  #endif
 #endif
@@ -99,7 +110,6 @@
 #if JUCE_MAC || JUCE_IOS
  #include <xlocale.h>
  #include <mach/mach.h>
- #include <signal.h>
 #endif
 
 #if JUCE_ANDROID
@@ -118,6 +128,7 @@
 //==============================================================================
 #include "containers/juce_AbstractFifo.cpp"
 #include "containers/juce_ArrayBase.cpp"
+#include "containers/juce_ListenerList.cpp"
 #include "containers/juce_NamedValueSet.cpp"
 #include "containers/juce_OwnedArray.cpp"
 #include "containers/juce_PropertySet.cpp"
@@ -196,6 +207,7 @@
  #include "native/juce_mac_Files.mm"
  #include "native/juce_mac_Network.mm"
  #include "native/juce_mac_Strings.mm"
+ #include "native/juce_intel_SharedCode.h"
  #include "native/juce_mac_SystemStats.mm"
  #include "native/juce_mac_Threads.mm"
 
@@ -208,19 +220,23 @@
  #include "native/juce_win32_Threads.cpp"
 
 //==============================================================================
-#elif JUCE_LINUX
+#elif JUCE_LINUX || JUCE_BSD
  #include "native/juce_linux_CommonFile.cpp"
  #include "native/juce_linux_Files.cpp"
  #include "native/juce_linux_Network.cpp"
  #if JUCE_USE_CURL
   #include "native/juce_curl_Network.cpp"
  #endif
+ #if JUCE_BSD
+  #include "native/juce_intel_SharedCode.h"
+ #endif
  #include "native/juce_linux_SystemStats.cpp"
  #include "native/juce_linux_Threads.cpp"
 
 //==============================================================================
 #elif JUCE_ANDROID
  #include "native/juce_linux_CommonFile.cpp"
+ #include "native/juce_android_AndroidDocument.cpp"
  #include "native/juce_android_JNIHelpers.cpp"
  #include "native/juce_android_Files.cpp"
  #include "native/juce_android_Misc.cpp"
@@ -229,18 +245,28 @@
  #include "native/juce_android_Threads.cpp"
  #include "native/juce_android_RuntimePermissions.cpp"
 
+#elif JUCE_WASM
+ #include "native/juce_wasm_SystemStats.cpp"
+
 #endif
 
-#include "threads/juce_ChildProcess.cpp"
+#include "files/juce_common_MimeTypes.h"
+#include "files/juce_common_MimeTypes.cpp"
 #include "threads/juce_HighResolutionTimer.cpp"
 #include "threads/juce_WaitableEvent.cpp"
 #include "network/juce_URL.cpp"
-#include "network/juce_WebInputStream.cpp"
-#include "streams/juce_URLInputSource.cpp"
+
+#if ! JUCE_WASM
+ #include "threads/juce_ChildProcess.cpp"
+ #include "network/juce_WebInputStream.cpp"
+ #include "streams/juce_URLInputSource.cpp"
+#endif
 
 //==============================================================================
 #if JUCE_UNIT_TESTS
  #include "containers/juce_HashMap_test.cpp"
+
+ #include "containers/juce_Optional_test.cpp"
 #endif
 
 //==============================================================================
