@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -23,7 +23,7 @@
 namespace juce
 {
 
-#if JUCE_MSVC
+#if JUCE_MSVC && ! defined (__INTEL_COMPILER)
  #pragma intrinsic (__cpuid)
  #pragma intrinsic (__rdtsc)
 #endif
@@ -62,11 +62,7 @@ static void callCPUID (int result[4], uint32 type)
 #else
 static void callCPUID (int result[4], int infoType)
 {
-   #if JUCE_PROJUCER_LIVE_BUILD
-    std::fill (result, result + 4, 0);
-   #else
     __cpuid (result, infoType);
-   #endif
 }
 #endif
 
@@ -150,7 +146,10 @@ void CPUInformation::initialise() noexcept
     hasSSSE3 = (info[2] & (1 <<  9)) != 0;
     hasSSE41 = (info[2] & (1 << 19)) != 0;
     hasSSE42 = (info[2] & (1 << 20)) != 0;
+
+    JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wshift-sign-overflow")
     has3DNow = (info[1] & (1 << 31)) != 0;
+    JUCE_END_IGNORE_WARNINGS_GCC_LIKE
 
     callCPUID (info, 0x80000001);
     hasFMA4  = (info[2] & (1 << 16)) != 0;
@@ -215,6 +214,7 @@ static DebugFlagsInitialiser debugFlagsInitialiser;
      return 0;
  }
 #else
+ RTL_OSVERSIONINFOW getWindowsVersionInfo();
  RTL_OSVERSIONINFOW getWindowsVersionInfo()
  {
      RTL_OSVERSIONINFOW versionInfo = {};
@@ -291,8 +291,12 @@ String SystemStats::getOperatingSystemName()
         case MacOSX_10_12:      JUCE_FALLTHROUGH
         case MacOSX_10_13:      JUCE_FALLTHROUGH
         case MacOSX_10_14:      JUCE_FALLTHROUGH
+        case MacOSX_10_15:      JUCE_FALLTHROUGH
+        case MacOS_11:          JUCE_FALLTHROUGH
+        case MacOS_12:          JUCE_FALLTHROUGH
 
         case UnknownOS:         JUCE_FALLTHROUGH
+        case WASM:              JUCE_FALLTHROUGH
         default:                jassertfalse; break; // !! new type of OS?
     }
 
@@ -328,8 +332,16 @@ bool SystemStats::isOperatingSystem64Bit()
    #else
     typedef BOOL (WINAPI* LPFN_ISWOW64PROCESS) (HANDLE, PBOOL);
 
+    const auto moduleHandle = GetModuleHandleA ("kernel32");
+
+    if (moduleHandle == nullptr)
+    {
+        jassertfalse;
+        return false;
+    }
+
     LPFN_ISWOW64PROCESS fnIsWow64Process
-        = (LPFN_ISWOW64PROCESS) GetProcAddress (GetModuleHandleA ("kernel32"), "IsWow64Process");
+        = (LPFN_ISWOW64PROCESS) GetProcAddress (moduleHandle, "IsWow64Process");
 
     BOOL isWow64 = FALSE;
 
@@ -397,7 +409,7 @@ public:
         LARGE_INTEGER f;
         QueryPerformanceFrequency (&f);
         hiResTicksPerSecond = f.QuadPart;
-        hiResTicksScaleFactor = 1000.0 / hiResTicksPerSecond;
+        hiResTicksScaleFactor = 1000.0 / (double) hiResTicksPerSecond;
     }
 
     inline int64 getHighResolutionTicks() noexcept
@@ -409,7 +421,7 @@ public:
 
     inline double getMillisecondCounterHiRes() noexcept
     {
-        return getHighResolutionTicks() * hiResTicksScaleFactor;
+        return (double) getHighResolutionTicks() * hiResTicksScaleFactor;
     }
 
     int64 hiResTicksPerSecond, hiResTicksOffset;
@@ -544,20 +556,36 @@ String SystemStats::getUserRegion()       { return getLocaleValue (LOCALE_USER_D
 String SystemStats::getDisplayLanguage()
 {
     DynamicLibrary dll ("kernel32.dll");
-    JUCE_LOAD_WINAPI_FUNCTION (dll, GetUserDefaultUILanguage, getUserDefaultUILanguage, LANGID, (void))
+    JUCE_LOAD_WINAPI_FUNCTION (dll,
+                               GetUserPreferredUILanguages,
+                               getUserPreferredUILanguages,
+                               BOOL,
+                               (DWORD, PULONG, PZZWSTR, PULONG))
 
-    if (getUserDefaultUILanguage == nullptr)
-        return "en";
+    constexpr auto defaultResult = "en";
 
-    auto langID = MAKELCID (getUserDefaultUILanguage(), SORT_DEFAULT);
+    if (getUserPreferredUILanguages == nullptr)
+        return defaultResult;
 
-    auto mainLang = getLocaleValue (langID, LOCALE_SISO639LANGNAME, "en");
-    auto region   = getLocaleValue (langID, LOCALE_SISO3166CTRYNAME, nullptr);
+    ULONG numLanguages = 0;
+    ULONG numCharsInLanguagesBuffer = 0;
 
-    if (region.isNotEmpty())
-        mainLang << '-' << region;
+    // Retrieving the necessary buffer size for storing the list of languages
+    if (! getUserPreferredUILanguages (MUI_LANGUAGE_NAME, &numLanguages, nullptr, &numCharsInLanguagesBuffer))
+        return defaultResult;
 
-    return mainLang;
+    std::vector<WCHAR> languagesBuffer (numCharsInLanguagesBuffer);
+    const auto success = getUserPreferredUILanguages (MUI_LANGUAGE_NAME,
+                                                      &numLanguages,
+                                                      languagesBuffer.data(),
+                                                      &numCharsInLanguagesBuffer);
+
+    if (! success || numLanguages == 0)
+        return defaultResult;
+
+    // The buffer contains a zero delimited list of languages, the first being
+    // the currently displayed language.
+    return languagesBuffer.data();
 }
 
 } // namespace juce
