@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -106,29 +106,26 @@ struct FallbackDownloadTask  : public URL::DownloadTask,
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (FallbackDownloadTask)
 };
 
-void URL::DownloadTask::Listener::progress (DownloadTask*, int64, int64) {}
-URL::DownloadTask::Listener::~Listener() {}
+void URL::DownloadTaskListener::progress (DownloadTask*, int64, int64) {}
 
 //==============================================================================
 std::unique_ptr<URL::DownloadTask> URL::DownloadTask::createFallbackDownloader (const URL& urlToUse,
                                                                                 const File& targetFileToUse,
-                                                                                const String& extraHeadersToUse,
-                                                                                Listener* listenerToUse,
-                                                                                bool usePostRequest)
+                                                                                const DownloadTaskOptions& options)
 {
     const size_t bufferSize = 0x8000;
     targetFileToUse.deleteFile();
 
     if (auto outputStream = targetFileToUse.createOutputStream (bufferSize))
     {
-        auto stream = std::make_unique<WebInputStream> (urlToUse, usePostRequest);
-        stream->withExtraHeaders (extraHeadersToUse);
+        auto stream = std::make_unique<WebInputStream> (urlToUse, options.usePost);
+        stream->withExtraHeaders (options.extraHeaders);
 
         if (stream->connect (nullptr))
             return std::make_unique<FallbackDownloadTask> (std::move (outputStream),
                                                            bufferSize,
                                                            std::move (stream),
-                                                           listenerToUse);
+                                                           options.listener);
     }
 
     return nullptr;
@@ -377,6 +374,11 @@ String URL::getFileName() const
 }
 #endif
 
+URL::ParameterHandling URL::toHandling (bool usePostData)
+{
+    return usePostData ? ParameterHandling::inPostData : ParameterHandling::inAddress;
+}
+
 File URL::fileFromFileSchemeURL (const URL& fileURL)
 {
     if (! fileURL.isLocalFile())
@@ -447,14 +449,21 @@ URL URL::getChildURL (const String& subPath) const
     return u;
 }
 
-void URL::createHeadersAndPostData (String& headers, MemoryBlock& postDataToWrite) const
+bool URL::hasBodyDataToSend() const
+{
+    return filesToUpload.size() > 0 || ! postData.isEmpty();
+}
+
+void URL::createHeadersAndPostData (String& headers,
+                                    MemoryBlock& postDataToWrite,
+                                    bool addParametersToBody) const
 {
     MemoryOutputStream data (postDataToWrite, false);
 
     if (filesToUpload.size() > 0)
     {
         // (this doesn't currently support mixing custom post-data with uploads..)
-        jassert (postData.getSize() == 0);
+        jassert (postData.isEmpty());
 
         auto boundary = String::toHexString (Random::getSystemRandom().nextInt64());
 
@@ -491,8 +500,10 @@ void URL::createHeadersAndPostData (String& headers, MemoryBlock& postDataToWrit
     }
     else
     {
-        data << URLHelpers::getMangledParameters (*this)
-             << postData;
+        if (addParametersToBody)
+            data << URLHelpers::getMangledParameters (*this);
+
+        data << postData;
 
         // if the user-supplied headers didn't contain a content-type, add one now..
         if (! headers.containsIgnoreCase ("Content-Type"))
@@ -656,81 +667,136 @@ private:
     }
 };
 #endif
+//==============================================================================
+template <typename Member, typename Item>
+static URL::InputStreamOptions with (URL::InputStreamOptions options, Member&& member, Item&& item)
+{
+    options.*member = std::forward<Item> (item);
+    return options;
+}
+
+URL::InputStreamOptions::InputStreamOptions (ParameterHandling handling)  : parameterHandling (handling)  {}
+
+URL::InputStreamOptions URL::InputStreamOptions::withProgressCallback (std::function<bool (int, int)> cb) const
+{
+    return with (*this, &InputStreamOptions::progressCallback, std::move (cb));
+}
+
+URL::InputStreamOptions URL::InputStreamOptions::withExtraHeaders (const String& headers) const
+{
+    return with (*this, &InputStreamOptions::extraHeaders, headers);
+}
+
+URL::InputStreamOptions URL::InputStreamOptions::withConnectionTimeoutMs (int timeout) const
+{
+    return with (*this, &InputStreamOptions::connectionTimeOutMs, timeout);
+}
+
+URL::InputStreamOptions URL::InputStreamOptions::withResponseHeaders (StringPairArray* headers) const
+{
+    return with (*this, &InputStreamOptions::responseHeaders, headers);
+}
+
+URL::InputStreamOptions URL::InputStreamOptions::withStatusCode (int* status) const
+{
+    return with (*this, &InputStreamOptions::statusCode, status);
+}
+
+URL::InputStreamOptions URL::InputStreamOptions::withNumRedirectsToFollow (int numRedirects) const
+{
+    return with (*this, &InputStreamOptions::numRedirectsToFollow, numRedirects);
+}
+
+URL::InputStreamOptions URL::InputStreamOptions::withHttpRequestCmd (const String& cmd) const
+{
+    return with (*this, &InputStreamOptions::httpRequestCmd, cmd);
+}
 
 //==============================================================================
-std::unique_ptr<InputStream> URL::createInputStream (bool usePostCommand,
-                                                     OpenStreamProgressCallback* progressCallback,
-                                                     void* progressCallbackContext,
-                                                     String headers,
-                                                     int timeOutMs,
-                                                     StringPairArray* responseHeaders,
-                                                     int* statusCode,
-                                                     int numRedirectsToFollow,
-                                                     String httpRequestCmd) const
+std::unique_ptr<InputStream> URL::createInputStream (const InputStreamOptions& options) const
 {
     if (isLocalFile())
     {
        #if JUCE_IOS
         // We may need to refresh the embedded bookmark.
-        return std::make_unique<iOSFileStreamWrapper<FileInputStream>> (const_cast<URL&>(*this));
+        return std::make_unique<iOSFileStreamWrapper<FileInputStream>> (const_cast<URL&> (*this));
        #else
         return getLocalFile().createInputStream();
        #endif
     }
 
-    auto wi = std::make_unique<WebInputStream> (*this, usePostCommand);
+    auto webInputStream = [&]
+    {
+        const auto usePost = options.getParameterHandling() == ParameterHandling::inPostData;
+        auto stream = std::make_unique<WebInputStream> (*this, usePost);
+
+        auto extraHeaders = options.getExtraHeaders();
+
+        if (extraHeaders.isNotEmpty())
+            stream->withExtraHeaders (extraHeaders);
+
+        auto timeout = options.getConnectionTimeoutMs();
+
+        if (timeout != 0)
+            stream->withConnectionTimeout (timeout);
+
+        auto requestCmd = options.getHttpRequestCmd();
+
+        if (requestCmd.isNotEmpty())
+            stream->withCustomRequestCommand (requestCmd);
+
+        stream->withNumRedirectsToFollow (options.getNumRedirectsToFollow());
+
+        return stream;
+    }();
 
     struct ProgressCallbackCaller  : public WebInputStream::Listener
     {
-        ProgressCallbackCaller (OpenStreamProgressCallback* progressCallbackToUse, void* progressCallbackContextToUse)
-            : callback (progressCallbackToUse), data (progressCallbackContextToUse)
-        {}
+        ProgressCallbackCaller (std::function<bool (int, int)> progressCallbackToUse)
+            : callback (std::move (progressCallbackToUse))
+        {
+        }
 
         bool postDataSendProgress (WebInputStream&, int bytesSent, int totalBytes) override
         {
-            return callback (data, bytesSent, totalBytes);
+            return callback (bytesSent, totalBytes);
         }
 
-        OpenStreamProgressCallback* callback;
-        void* const data;
+        std::function<bool (int, int)> callback;
     };
 
-    std::unique_ptr<ProgressCallbackCaller> callbackCaller
-        (progressCallback != nullptr ? new ProgressCallbackCaller (progressCallback, progressCallbackContext) : nullptr);
+    auto callbackCaller = [&options]() -> std::unique_ptr<ProgressCallbackCaller>
+    {
+        if (auto progressCallback = options.getProgressCallback())
+            return std::make_unique<ProgressCallbackCaller> (progressCallback);
 
-    if (headers.isNotEmpty())
-        wi->withExtraHeaders (headers);
+        return {};
+    }();
 
-    if (timeOutMs != 0)
-        wi->withConnectionTimeout (timeOutMs);
+    auto success = webInputStream->connect (callbackCaller.get());
 
-    if (httpRequestCmd.isNotEmpty())
-        wi->withCustomRequestCommand (httpRequestCmd);
+    if (auto* status = options.getStatusCode())
+        *status = webInputStream->getStatusCode();
 
-    wi->withNumRedirectsToFollow (numRedirectsToFollow);
+    if (auto* responseHeaders = options.getResponseHeaders())
+        *responseHeaders = webInputStream->getResponseHeaders();
 
-    bool success = wi->connect (callbackCaller.get());
-
-    if (statusCode != nullptr)
-        *statusCode = wi->getStatusCode();
-
-    if (responseHeaders != nullptr)
-        *responseHeaders = wi->getResponseHeaders();
-
-    if (! success || wi->isError())
+    if (! success || webInputStream->isError())
         return nullptr;
 
-    // Older GCCs complain about binding unique_ptr<WebInputStream>&& to unique_ptr<InputStream>
-    // if we just `return wi` here.
-    return std::unique_ptr<InputStream> (std::move (wi));
+    // std::move() needed here for older compilers
+    JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wredundant-move")
+    return std::move (webInputStream);
+    JUCE_END_IGNORE_WARNINGS_GCC_LIKE
 }
 
-#if JUCE_ANDROID
-OutputStream* juce_CreateContentURIOutputStream (const URL&);
-#endif
-
 std::unique_ptr<OutputStream> URL::createOutputStream() const
 {
+   #if JUCE_ANDROID
+    if (auto stream = AndroidDocument::fromDocument (*this).createOutputStream())
+        return stream;
+   #endif
+
     if (isLocalFile())
     {
        #if JUCE_IOS
@@ -741,18 +807,14 @@ std::unique_ptr<OutputStream> URL::createOutputStream() const
        #endif
     }
 
-   #if JUCE_ANDROID
-    return std::unique_ptr<OutputStream> (juce_CreateContentURIOutputStream (*this));
-   #else
     return nullptr;
-   #endif
 }
 
 //==============================================================================
 bool URL::readEntireBinaryStream (MemoryBlock& destData, bool usePostCommand) const
 {
     const std::unique_ptr<InputStream> in (isLocalFile() ? getLocalFile().createInputStream()
-                                                         : createInputStream (usePostCommand));
+                                                         : createInputStream (InputStreamOptions (toHandling (usePostCommand))));
 
     if (in != nullptr)
     {
@@ -766,7 +828,7 @@ bool URL::readEntireBinaryStream (MemoryBlock& destData, bool usePostCommand) co
 String URL::readEntireTextStream (bool usePostCommand) const
 {
     const std::unique_ptr<InputStream> in (isLocalFile() ? getLocalFile().createInputStream()
-                                                         : createInputStream (usePostCommand));
+                                                         : createInputStream (InputStreamOptions (toHandling (usePostCommand))));
 
     if (in != nullptr)
         return in->readEntireStreamAsString();
@@ -823,7 +885,7 @@ URL URL::withUpload (Upload* const f) const
     auto u = *this;
 
     for (int i = u.filesToUpload.size(); --i >= 0;)
-        if (u.filesToUpload.getObjectPointerUnchecked(i)->parameterName == f->parameterName)
+        if (u.filesToUpload.getObjectPointerUnchecked (i)->parameterName == f->parameterName)
             u.filesToUpload.remove (i);
 
     u.filesToUpload.add (f);
@@ -911,4 +973,41 @@ bool URL::launchInDefaultBrowser() const
     return Process::openDocument (u, {});
 }
 
+//==============================================================================
+std::unique_ptr<InputStream> URL::createInputStream (bool usePostCommand,
+                                                     OpenStreamProgressCallback* cb,
+                                                     void* context,
+                                                     String headers,
+                                                     int timeOutMs,
+                                                     StringPairArray* responseHeaders,
+                                                     int* statusCode,
+                                                     int numRedirectsToFollow,
+                                                     String httpRequestCmd) const
+{
+    std::function<bool (int, int)> callback;
+
+    if (cb != nullptr)
+        callback = [context, cb] (int sent, int total) { return cb (context, sent, total); };
+
+    return createInputStream (InputStreamOptions (toHandling (usePostCommand))
+                                .withProgressCallback (std::move (callback))
+                                .withExtraHeaders (headers)
+                                .withConnectionTimeoutMs (timeOutMs)
+                                .withResponseHeaders (responseHeaders)
+                                .withStatusCode (statusCode)
+                                .withNumRedirectsToFollow(numRedirectsToFollow)
+                                .withHttpRequestCmd (httpRequestCmd));
+}
+
+std::unique_ptr<URL::DownloadTask> URL::downloadToFile (const File& targetLocation,
+                                                        String extraHeaders,
+                                                        DownloadTask::Listener* listener,
+                                                        bool usePostCommand)
+{
+    auto options = DownloadTaskOptions().withExtraHeaders (std::move (extraHeaders))
+                                        .withListener (listener)
+                                        .withUsePost (usePostCommand);
+    return downloadToFile (targetLocation, std::move (options));
+}
+
 } // namespace juce
