@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -20,7 +20,7 @@
   ==============================================================================
 */
 
-#if ! DOXYGEN && (JUCE_MAC || JUCE_IOS)
+#if ! defined (DOXYGEN) && (JUCE_MAC || JUCE_IOS)
  // Annoyingly we can only forward-declare a typedef by forward-declaring the
  // aliased type
  #if __has_attribute(objc_bridge)
@@ -1049,7 +1049,8 @@ public:
         This will look for a value at the end of the string.
         e.g. for "321 xyz654" it will return 654; for "2 3 4" it'll return 4.
 
-        Negative numbers are not handled, so "xyz-5" returns 5.
+        If the string ends with a hyphen followed by numeric characters, the
+        return value will be negative.
 
         @see getIntValue
     */
@@ -1133,94 +1134,6 @@ public:
 
         auto numDigitsBeforePoint = (int) std::ceil (std::log10 (number < 0 ? -number : number));
 
-       #if JUCE_PROJUCER_LIVE_BUILD
-        auto doubleNumber = (double) number;
-        constexpr int bufferSize = 311;
-        char buffer[bufferSize];
-        auto* ptr = &(buffer[0]);
-        auto* const safeEnd = ptr + (bufferSize - 1);
-        auto numSigFigsParsed = 0;
-
-        auto writeToBuffer = [safeEnd] (char* destination, char data)
-        {
-            *destination++ = data;
-
-            if (destination == safeEnd)
-            {
-                *destination = '\0';
-                return true;
-            }
-
-            return false;
-        };
-
-        auto truncateOrRound = [numberOfSignificantFigures] (double fractional, int sigFigsParsed)
-        {
-            return (sigFigsParsed == numberOfSignificantFigures - 1) ? (int) std::round (fractional)
-                                                                     : (int) fractional;
-        };
-
-        if (doubleNumber < 0)
-        {
-            doubleNumber *= -1;
-            *ptr++ = '-';
-        }
-
-        if (numDigitsBeforePoint > 0)
-        {
-            doubleNumber /= std::pow (10.0, numDigitsBeforePoint);
-
-            while (numDigitsBeforePoint-- > 0)
-            {
-                if (numSigFigsParsed == numberOfSignificantFigures)
-                {
-                    if (writeToBuffer (ptr++, '0'))
-                        return buffer;
-
-                    continue;
-                }
-
-                doubleNumber *= 10;
-                auto digit = truncateOrRound (doubleNumber, numSigFigsParsed);
-
-                if (writeToBuffer (ptr++, (char) ('0' + digit)))
-                    return buffer;
-
-                ++numSigFigsParsed;
-                doubleNumber -= digit;
-            }
-
-            if (numSigFigsParsed == numberOfSignificantFigures)
-            {
-                *ptr++ = '\0';
-                return buffer;
-            }
-        }
-        else
-        {
-            *ptr++ = '0';
-        }
-
-        if (writeToBuffer (ptr++, '.'))
-            return buffer;
-
-        while (numSigFigsParsed < numberOfSignificantFigures)
-        {
-            doubleNumber *= 10;
-            auto digit = truncateOrRound (doubleNumber, numSigFigsParsed);
-
-            if (writeToBuffer (ptr++, (char) ('0' + digit)))
-                return buffer;
-
-            if (numSigFigsParsed != 0 || digit != 0)
-                ++numSigFigsParsed;
-
-            doubleNumber -= digit;
-        }
-
-        *ptr++ = '\0';
-        return buffer;
-       #else
         auto shift = numberOfSignificantFigures - numDigitsBeforePoint;
         auto factor = std::pow (10.0, shift);
         auto rounded = std::round (number * factor) / factor;
@@ -1228,7 +1141,6 @@ public:
         std::stringstream ss;
         ss << std::fixed << std::setprecision (std::max (shift, 0)) << rounded;
         return ss.str();
-       #endif
     }
 
     //==============================================================================
@@ -1414,15 +1326,13 @@ public:
     int getReferenceCount() const noexcept;
 
     //==============================================================================
-    /*  This was a static empty string object, but is now deprecated as it's too easy to accidentally
-        use it indirectly during a static constructor, leading to hard-to-find order-of-initialisation
-        problems.
-        @deprecated If you need an empty String object, just use String() or {}.
-        The only time you might miss having String::empty available might be if you need to return an
-        empty string from a function by reference, but if you need to do that, it's easy enough to use
-        a function-local static String object and return that, avoiding any order-of-initialisation issues.
-    */
-    JUCE_DEPRECATED_STATIC (static const String empty;)
+   #if JUCE_ALLOW_STATIC_NULL_VARIABLES && ! defined (DOXYGEN)
+    [[deprecated ("This was a static empty string object, but is now deprecated as it's too easy to accidentally "
+                 "use it indirectly during a static constructor, leading to hard-to-find order-of-initialisation "
+                 "problems. If you need an empty String object, just use String() or {}. For returning an empty "
+                 "String from a function by reference, use a function-local static String object and return that.")]]
+    static const String empty;
+   #endif
 
 private:
     //==============================================================================
@@ -1437,7 +1347,6 @@ private:
 
     explicit String (const PreallocationBytes&); // This constructor preallocates a certain amount of memory
     size_t getByteOffsetOfEnd() const noexcept;
-    JUCE_DEPRECATED (String (const String&, size_t));
 
     // This private cast operator should prevent strings being accidentally cast
     // to bools (this is possible because the compiler can add an implicit cast
@@ -1587,7 +1496,7 @@ JUCE_API OutputStream& JUCE_CALLTYPE operator<< (OutputStream& stream, StringRef
 
 } // namespace juce
 
-#if ! DOXYGEN
+#ifndef DOXYGEN
 namespace std
 {
     template <> struct hash<juce::String>
