@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -81,7 +81,7 @@ String ChildProcess::readAllProcessOutput()
 }
 
 
-uint32 ChildProcess::getPID() const noexcept
+int ChildProcess::getPID() const noexcept
 {
     return activeProcess != nullptr ? activeProcess->getPID() : 0;
 }
@@ -101,7 +101,7 @@ public:
     {
         beginTest ("Child Processes");
 
-      #if JUCE_WINDOWS || JUCE_MAC || JUCE_LINUX
+      #if JUCE_WINDOWS || JUCE_MAC || JUCE_LINUX || JUCE_BSD
         ChildProcess p;
 
        #if JUCE_WINDOWS
