@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -26,10 +26,6 @@
 namespace juce
 {
 
-#if MAC_OS_X_VERSION_MIN_REQUIRED <= MAC_OS_X_VERSION_10_8
- #define __nullable
-#endif
-
 //==============================================================================
 // This class has been renamed from CoreGraphicsImage to avoid a symbol
 // collision in Pro Tools 2019.12 and possibly 2020 depending on the Pro Tools
@@ -52,12 +48,12 @@ public:
         numComponents += (size_t) lineStride;
        #endif
 
-        imageDataHolder->data.allocate (numComponents, clearImage);
+        imageData->data.allocate (numComponents, clearImage);
 
         auto colourSpace = detail::ColorSpacePtr { CGColorSpaceCreateWithName ((format == Image::SingleChannel) ? kCGColorSpaceGenericGrayGamma2_2
                                                                                                                 : kCGColorSpaceSRGB) };
 
-        context = detail::ContextPtr { CGBitmapContextCreate (imageDataHolder->data, (size_t) width, (size_t) height, 8, (size_t) lineStride,
+        context = detail::ContextPtr { CGBitmapContextCreate (imageData->data, (size_t) width, (size_t) height, 8, (size_t) lineStride,
                                                               colourSpace.get(), getCGImageFlags (format)) };
     }
 
@@ -75,7 +71,9 @@ public:
 
     void initialiseBitmapData (Image::BitmapData& bitmap, int x, int y, Image::BitmapData::ReadWriteMode mode) override
     {
-        bitmap.data = imageDataHolder->data + x * pixelStride + y * lineStride;
+        const auto offset = (size_t) (x * pixelStride + y * lineStride);
+        bitmap.data = imageData->data + offset;
+        bitmap.size = (size_t) (lineStride * height) - offset;
         bitmap.pixelFormat = pixelFormat;
         bitmap.lineStride = lineStride;
         bitmap.pixelStride = pixelStride;
@@ -90,7 +88,7 @@ public:
     ImagePixelData::Ptr clone() override
     {
         auto im = new CoreGraphicsPixelData (pixelFormat, width, height, false);
-        memcpy (im->imageDataHolder->data, imageDataHolder->data, (size_t) (lineStride * height));
+        memcpy (im->imageData->data, imageData->data, (size_t) (lineStride * height));
         return *im;
     }
 
@@ -102,12 +100,9 @@ public:
         auto cgim = dynamic_cast<CoreGraphicsPixelData*> (juceImage.getPixelData());
 
         if (cgim != nullptr && cgim->cachedImageRef != nullptr)
-        {
-            CGImageRetain (cgim->cachedImageRef.get());
-            return cgim->cachedImageRef.get();
-        }
+            return CGImageRetain (cgim->cachedImageRef.get());
 
-        CGImageRef ref = createImage (juceImage, colourSpace, false);
+        CGImageRef ref = createImage (juceImage, colourSpace);
 
         if (cgim != nullptr)
             cgim->cachedImageRef.reset (CGImageRetain (ref));
@@ -115,55 +110,47 @@ public:
         return ref;
     }
 
-    static CGImageRef createImage (const Image& juceImage, CGColorSpaceRef colourSpace, bool mustOutliveSource)
+    static CGImageRef createImage (const Image& juceImage, CGColorSpaceRef colourSpace)
     {
         const Image::BitmapData srcData (juceImage, Image::BitmapData::readOnly);
-        detail::DataProviderPtr provider;
 
-        if (mustOutliveSource)
+        const auto provider = [&]
         {
-            CFDataRef data = CFDataCreate (nullptr, (const UInt8*) srcData.data, (CFIndex) ((size_t) srcData.lineStride * (size_t) srcData.height));
-            provider = detail::DataProviderPtr { CGDataProviderCreateWithCFData (data) };
-            CFRelease (data);
-        }
-        else
-        {
-            auto* imageDataContainer = [] (const Image& img) -> HeapBlockContainer::Ptr*
+            if (auto* cgim = dynamic_cast<CoreGraphicsPixelData*> (juceImage.getPixelData()))
             {
-                if (auto* cgim = dynamic_cast<CoreGraphicsPixelData*> (img.getPixelData()))
-                    return new HeapBlockContainer::Ptr (cgim->imageDataHolder);
-
-                return nullptr;
-            } (juceImage);
-
-            provider = detail::DataProviderPtr { CGDataProviderCreateWithData (imageDataContainer,
+                return detail::DataProviderPtr { CGDataProviderCreateWithData (new ImageDataContainer::Ptr (cgim->imageData),
                                                                                srcData.data,
-                                                                               (size_t) srcData.lineStride * (size_t) srcData.height,
-                                                                               [] (void * __nullable info, const void*, size_t) { delete (HeapBlockContainer::Ptr*) info; }) };
-        }
+                                                                               srcData.size,
+                                                                               [] (void * __nullable info, const void*, size_t) { delete (ImageDataContainer::Ptr*) info; }) };
+            }
 
-        CGImageRef imageRef = CGImageCreate ((size_t) srcData.width,
-                                             (size_t) srcData.height,
-                                             8,
-                                             (size_t) srcData.pixelStride * 8,
-                                             (size_t) srcData.lineStride,
-                                             colourSpace, getCGImageFlags (juceImage.getFormat()), provider.get(),
-                                             nullptr, true, kCGRenderingIntentDefault);
+            const auto usableSize = jmin ((size_t) srcData.lineStride * (size_t) srcData.height, srcData.size);
+            CFUniquePtr<CFDataRef> data (CFDataCreate (nullptr, (const UInt8*) srcData.data, (CFIndex) usableSize));
+            return detail::DataProviderPtr { CGDataProviderCreateWithCFData (data.get()) };
+        }();
 
-        return imageRef;
+        return CGImageCreate ((size_t) srcData.width,
+                              (size_t) srcData.height,
+                              8,
+                              (size_t) srcData.pixelStride * 8,
+                              (size_t) srcData.lineStride,
+                              colourSpace, getCGImageFlags (juceImage.getFormat()), provider.get(),
+                              nullptr, true, kCGRenderingIntentDefault);
     }
 
     //==============================================================================
     detail::ContextPtr context;
     detail::ImagePtr cachedImageRef;
 
-    struct HeapBlockContainer   : public ReferenceCountedObject
+    struct ImageDataContainer   : public ReferenceCountedObject
     {
-        using Ptr = ReferenceCountedObjectPtr<HeapBlockContainer>;
+        ImageDataContainer() = default;
+
+        using Ptr = ReferenceCountedObjectPtr<ImageDataContainer>;
         HeapBlock<uint8> data;
     };
 
-    HeapBlockContainer::Ptr imageDataHolder = new HeapBlockContainer();
+    ImageDataContainer::Ptr imageData = new ImageDataContainer();
     int pixelStride, lineStride;
 
 private:
@@ -175,9 +162,9 @@ private:
     static CGBitmapInfo getCGImageFlags (const Image::PixelFormat& format)
     {
        #if JUCE_BIG_ENDIAN
-        return format == Image::ARGB ? (kCGImageAlphaPremultipliedFirst | kCGBitmapByteOrder32Big) : kCGBitmapByteOrderDefault;
+        return format == Image::ARGB ? ((uint32_t) kCGImageAlphaPremultipliedFirst | (uint32_t) kCGBitmapByteOrder32Big) : kCGBitmapByteOrderDefault;
        #else
-        return format == Image::ARGB ? (kCGImageAlphaPremultipliedFirst | kCGBitmapByteOrder32Little) : kCGBitmapByteOrderDefault;
+        return format == Image::ARGB ? ((uint32_t) kCGImageAlphaPremultipliedFirst | (uint32_t) kCGBitmapByteOrder32Little) : kCGBitmapByteOrderDefault;
        #endif
     }
 
@@ -332,7 +319,7 @@ void CoreGraphicsContext::clipToImageAlpha (const Image& sourceImage, const Affi
         if (sourceImage.getFormat() != Image::SingleChannel)
             singleChannelImage = sourceImage.convertedToFormat (Image::SingleChannel);
 
-        auto image = detail::ImagePtr { CoreGraphicsPixelData::createImage (singleChannelImage, greyColourSpace.get(), true) };
+        auto image = detail::ImagePtr { CoreGraphicsPixelData::createImage (singleChannelImage, greyColourSpace.get()) };
 
         flip();
         auto t = AffineTransform::verticalFlip (sourceImage.getHeight()).followedBy (transform);
@@ -524,7 +511,7 @@ void CoreGraphicsContext::drawImage (const Image& sourceImage, const AffineTrans
 
     auto colourSpace = sourceImage.getFormat() == Image::PixelFormat::SingleChannel ? greyColourSpace.get()
                                                                                     : rgbColourSpace.get();
-    auto image = detail::ImagePtr { CoreGraphicsPixelData::getCachedImageRef (sourceImage, colourSpace) };
+    detail::ImagePtr image { CoreGraphicsPixelData::getCachedImageRef (sourceImage, colourSpace) };
 
     ScopedCGContextState scopedState (context.get());
     CGContextSetAlpha (context.get(), state->fillType.getOpacity());
@@ -613,7 +600,9 @@ void CoreGraphicsContext::setFont (const Font& newFont)
         state->fontRef = nullptr;
         state->font = newFont;
 
-        if (auto osxTypeface = dynamic_cast<OSXTypeface*> (state->font.getTypeface()))
+        auto typeface = state->font.getTypefacePtr();
+
+        if (auto osxTypeface = dynamic_cast<OSXTypeface*> (typeface.get()))
         {
             state->fontRef = osxTypeface->fontRef;
             CGContextSetFont (context.get(), state->fontRef);
@@ -670,7 +659,7 @@ void CoreGraphicsContext::drawGlyph (int glyphNumber, const AffineTransform& tra
     {
         Path p;
         auto& f = state->font;
-        f.getTypeface()->getOutlineForGlyph (glyphNumber, p);
+        f.getTypefacePtr()->getOutlineForGlyph (glyphNumber, p);
 
         fillPath (p, AffineTransform::scale (f.getHeight() * f.getHorizontalScale(), f.getHeight())
                                      .followedBy (transform));
@@ -679,8 +668,7 @@ void CoreGraphicsContext::drawGlyph (int glyphNumber, const AffineTransform& tra
 
 bool CoreGraphicsContext::drawTextLayout (const AttributedString& text, const Rectangle<float>& area)
 {
-    CoreTextTypeLayout::drawToCGContext (text, area, context.get(), (float) flipHeight);
-    return true;
+    return CoreTextTypeLayout::drawToCGContext (text, area, context.get(), (float) flipHeight);
 }
 
 CoreGraphicsContext::SavedState::SavedState()
@@ -833,6 +821,9 @@ Image juce_loadWithCoreImage (InputStream& input)
     MemoryBlockHolder::Ptr memBlockHolder = new MemoryBlockHolder();
     input.readIntoMemoryBlock (memBlockHolder->block, -1);
 
+    if (memBlockHolder->block.isEmpty())
+        return {};
+
    #if JUCE_IOS
     JUCE_AUTORELEASEPOOL
    #endif
@@ -849,12 +840,11 @@ Image juce_loadWithCoreImage (InputStream& input)
                                                                                 memBlockHolder->block.getData(),
                                                                                 memBlockHolder->block.getSize(),
                                                                                 [] (void * __nullable info, const void*, size_t) { delete (MemoryBlockHolder::Ptr*) info; }) };
-        auto imageSource = CGImageSourceCreateWithDataProvider (provider.get(), nullptr);
 
-        if (imageSource != nullptr)
+        if (auto imageSource = CFUniquePtr<CGImageSourceRef> (CGImageSourceCreateWithDataProvider (provider.get(), nullptr)))
         {
-            auto loadedImage = CGImageSourceCreateImageAtIndex (imageSource, 0, nullptr);
-            CFRelease (imageSource);
+            CFUniquePtr<CGImageRef> loadedImagePtr (CGImageSourceCreateImageAtIndex (imageSource.get(), 0, nullptr));
+            auto* loadedImage = loadedImagePtr.get();
       #endif
 
             if (loadedImage != nullptr)
@@ -875,10 +865,6 @@ Image juce_loadWithCoreImage (InputStream& input)
                 CGContextDrawImage (cgImage->context.get(), convertToCGRect (image.getBounds()), loadedImage);
                 CGContextFlush (cgImage->context.get());
 
-               #if ! JUCE_IOS
-                CFRelease (loadedImage);
-               #endif
-
                 // Because it's impossible to create a truly 24-bit CG image, this flag allows a user
                 // to find out whether the file they just loaded the image from had an alpha channel or not.
                 image.getProperties()->set ("originalImageHadAlpha", hasAlphaChan);
@@ -903,10 +889,9 @@ Image juce_createImageFromCIImage (CIImage* im, int w, int h)
     return Image (*cgImage);
 }
 
-CGImageRef juce_createCoreGraphicsImage (const Image& juceImage, CGColorSpaceRef colourSpace,
-                                         const bool mustOutliveSource)
+CGImageRef juce_createCoreGraphicsImage (const Image& juceImage, CGColorSpaceRef colourSpace)
 {
-    return CoreGraphicsPixelData::createImage (juceImage, colourSpace, mustOutliveSource);
+    return CoreGraphicsPixelData::createImage (juceImage, colourSpace);
 }
 
 CGContextRef juce_getImageContext (const Image& image)
@@ -933,16 +918,19 @@ CGContextRef juce_getImageContext (const Image& image)
 #endif
 
 #if JUCE_MAC
- NSImage* imageToNSImage (const Image& image, float scaleFactor)
+ NSImage* imageToNSImage (const ScaledImage& scaled)
  {
+     const auto image = scaled.getImage();
+     const auto scaleFactor = scaled.getScale();
+
      JUCE_AUTORELEASEPOOL
      {
          NSImage* im = [[NSImage alloc] init];
          auto requiredSize = NSMakeSize (image.getWidth() / scaleFactor, image.getHeight() / scaleFactor);
 
          [im setSize: requiredSize];
-         auto colourSpace = detail::ColorSpacePtr { CGColorSpaceCreateWithName (kCGColorSpaceSRGB) };
-         auto imageRef = detail::ImagePtr { juce_createCoreGraphicsImage (image, colourSpace.get(), true) };
+         detail::ColorSpacePtr colourSpace { CGColorSpaceCreateWithName (kCGColorSpaceSRGB) };
+         detail::ImagePtr imageRef { juce_createCoreGraphicsImage (image, colourSpace.get()) };
 
          NSBitmapImageRep* imageRep = [[NSBitmapImageRep alloc] initWithCGImage: imageRef.get()];
          [imageRep setSize: requiredSize];
