@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -263,15 +263,17 @@ ApplicationCommandTarget* ApplicationCommandManager::findDefaultComponentTarget(
         }
     }
 
-    if (c == nullptr && Process::isForegroundProcess())
+    if (c == nullptr)
     {
         auto& desktop = Desktop::getInstance();
 
         // getting a bit desperate now: try all desktop comps..
         for (int i = desktop.getNumComponents(); --i >= 0;)
-            if (auto* peer = desktop.getComponent(i)->getPeer())
-                if (auto* target = findTargetForComponent (peer->getLastFocusedSubcomponent()))
-                    return target;
+            if (auto* component = desktop.getComponent (i))
+                if (isForegroundOrEmbeddedProcess (component))
+                    if (auto* peer = component->getPeer())
+                        if (auto* target = findTargetForComponent (peer->getLastFocusedSubcomponent()))
+                            return target;
     }
 
     if (c != nullptr)
