@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -48,7 +48,7 @@ void MultiDocumentPanelWindow::maximiseButtonPressed()
 void MultiDocumentPanelWindow::closeButtonPressed()
 {
     if (auto* owner = getOwner())
-        owner->closeDocument (getContentComponent(), true);
+        owner->closeDocumentAsync (getContentComponent(), true, nullptr);
     else
         jassertfalse; // these windows are only designed to be used inside a MultiDocumentPanel!
 }
@@ -97,7 +97,9 @@ MultiDocumentPanel::MultiDocumentPanel()
 
 MultiDocumentPanel::~MultiDocumentPanel()
 {
-    closeAllDocuments (false);
+    for (int i = components.size(); --i >= 0;)
+        if (auto* component = components[i])
+            closeDocumentInternal (component);
 }
 
 //==============================================================================
@@ -109,6 +111,7 @@ namespace MultiDocHelpers
     }
 }
 
+#if JUCE_MODAL_LOOPS_PERMITTED
 bool MultiDocumentPanel::closeAllDocuments (const bool checkItsOkToCloseFirst)
 {
     while (! components.isEmpty())
@@ -117,6 +120,52 @@ bool MultiDocumentPanel::closeAllDocuments (const bool checkItsOkToCloseFirst)
 
     return true;
 }
+#endif
+
+void MultiDocumentPanel::closeLastDocumentRecursive (SafePointer<MultiDocumentPanel> parent,
+                                                     bool checkItsOkToCloseFirst,
+                                                     std::function<void (bool)> callback)
+{
+    if (parent->components.isEmpty())
+    {
+        if (callback != nullptr)
+            callback (true);
+
+        return;
+    }
+
+    parent->closeDocumentAsync (parent->components.getLast(),
+                                checkItsOkToCloseFirst,
+                                [parent, checkItsOkToCloseFirst, callback] (bool closeResult)
+    {
+        if (parent == nullptr)
+            return;
+
+        if (! closeResult)
+        {
+            if (callback != nullptr)
+                callback (false);
+
+            return;
+        }
+
+        parent->closeLastDocumentRecursive (parent, checkItsOkToCloseFirst, std::move (callback));
+    });
+}
+
+void MultiDocumentPanel::closeAllDocumentsAsync (bool checkItsOkToCloseFirst, std::function<void (bool)> callback)
+{
+    closeLastDocumentRecursive (this, checkItsOkToCloseFirst, std::move (callback));
+}
+
+#if JUCE_MODAL_LOOPS_PERMITTED
+bool MultiDocumentPanel::tryToCloseDocument (Component*)
+{
+    // If you hit this assertion then you need to implement this method in a subclass.
+    jassertfalse;
+    return false;
+}
+#endif
 
 MultiDocumentPanelWindow* MultiDocumentPanel::createNewDocumentWindow()
 {
@@ -217,86 +266,105 @@ bool MultiDocumentPanel::addDocument (Component* const component,
     return true;
 }
 
+void MultiDocumentPanel::closeDocumentInternal (Component* component)
+{
+    // Intellisense warns about component being uninitialised.
+    // I'm not sure how a function argument could be uninitialised.
+    JUCE_BEGIN_IGNORE_WARNINGS_MSVC (6001)
+
+    component->removeComponentListener (this);
+
+    const bool shouldDelete = MultiDocHelpers::shouldDeleteComp (component);
+    component->getProperties().remove ("mdiDocumentDelete_");
+    component->getProperties().remove ("mdiDocumentBkg_");
+
+    if (mode == FloatingWindows)
+    {
+        for (auto* child : getChildren())
+        {
+            if (auto* dw = dynamic_cast<MultiDocumentPanelWindow*> (child))
+            {
+                if (dw->getContentComponent() == component)
+                {
+                    std::unique_ptr<MultiDocumentPanelWindow> (dw)->clearContentComponent();
+                    break;
+                }
+            }
+        }
+
+        if (shouldDelete)
+            delete component;
+
+        components.removeFirstMatchingValue (component);
+
+        if (isFullscreenWhenOneDocument() && components.size() == 1)
+        {
+            for (int i = getNumChildComponents(); --i >= 0;)
+            {
+                std::unique_ptr<MultiDocumentPanelWindow> dw (dynamic_cast<MultiDocumentPanelWindow*> (getChildComponent (i)));
+
+                if (dw != nullptr)
+                    dw->clearContentComponent();
+            }
+
+            addAndMakeVisible (components.getFirst());
+        }
+    }
+    else
+    {
+        jassert (components.indexOf (component) >= 0);
+
+        if (tabComponent != nullptr)
+        {
+            for (int i = tabComponent->getNumTabs(); --i >= 0;)
+                if (tabComponent->getTabContentComponent (i) == component)
+                    tabComponent->removeTab (i);
+        }
+        else
+        {
+            removeChildComponent (component);
+        }
+
+        if (shouldDelete)
+            delete component;
+
+        if (tabComponent != nullptr && tabComponent->getNumTabs() <= numDocsBeforeTabsUsed)
+            tabComponent.reset();
+
+        components.removeFirstMatchingValue (component);
+
+        if (components.size() > 0 && tabComponent == nullptr)
+            addAndMakeVisible (components.getFirst());
+    }
+
+    resized();
+
+    // This ensures that the active tab is painted properly when a tab is closed!
+    if (auto* activeComponent = getActiveDocument())
+        setActiveDocument (activeComponent);
+
+    activeDocumentChanged();
+
+    JUCE_END_IGNORE_WARNINGS_MSVC
+}
+
+#if JUCE_MODAL_LOOPS_PERMITTED
 bool MultiDocumentPanel::closeDocument (Component* component,
                                         const bool checkItsOkToCloseFirst)
 {
+    // Intellisense warns about component being uninitialised.
+    // I'm not sure how a function argument could be uninitialised.
+    JUCE_BEGIN_IGNORE_WARNINGS_MSVC (6001)
+
+    if (component == nullptr)
+        return true;
+
     if (components.contains (component))
     {
         if (checkItsOkToCloseFirst && ! tryToCloseDocument (component))
             return false;
 
-        component->removeComponentListener (this);
-
-        const bool shouldDelete = MultiDocHelpers::shouldDeleteComp (component);
-        component->getProperties().remove ("mdiDocumentDelete_");
-        component->getProperties().remove ("mdiDocumentBkg_");
-
-        if (mode == FloatingWindows)
-        {
-            for (auto* child : getChildren())
-            {
-                if (auto* dw = dynamic_cast<MultiDocumentPanelWindow*> (child))
-                {
-                    if (dw->getContentComponent() == component)
-                    {
-                        std::unique_ptr<MultiDocumentPanelWindow> (dw)->clearContentComponent();
-                        break;
-                    }
-                }
-            }
-
-            if (shouldDelete)
-                delete component;
-
-            components.removeFirstMatchingValue (component);
-
-            if (isFullscreenWhenOneDocument() && components.size() == 1)
-            {
-                for (int i = getNumChildComponents(); --i >= 0;)
-                {
-                    std::unique_ptr<MultiDocumentPanelWindow> dw (dynamic_cast<MultiDocumentPanelWindow*> (getChildComponent (i)));
-
-                    if (dw != nullptr)
-                        dw->clearContentComponent();
-                }
-
-                addAndMakeVisible (components.getFirst());
-            }
-        }
-        else
-        {
-            jassert (components.indexOf (component) >= 0);
-
-            if (tabComponent != nullptr)
-            {
-                for (int i = tabComponent->getNumTabs(); --i >= 0;)
-                    if (tabComponent->getTabContentComponent (i) == component)
-                        tabComponent->removeTab (i);
-            }
-            else
-            {
-                removeChildComponent (component);
-            }
-
-            if (shouldDelete)
-                delete component;
-
-            if (tabComponent != nullptr && tabComponent->getNumTabs() <= numDocsBeforeTabsUsed)
-                tabComponent.reset();
-
-            components.removeFirstMatchingValue (component);
-
-            if (components.size() > 0 && tabComponent == nullptr)
-                addAndMakeVisible (components.getFirst());
-        }
-
-        resized();
-
-        // This ensures that the active tab is painted properly when a tab is closed!
-        if (auto* activeComponent = getActiveDocument())
-            setActiveDocument (activeComponent);
-
-        activeDocumentChanged();
+        closeDocumentInternal (component);
     }
     else
     {
@@ -304,6 +372,58 @@ bool MultiDocumentPanel::closeDocument (Component* component,
     }
 
     return true;
+
+    JUCE_END_IGNORE_WARNINGS_MSVC
+}
+#endif
+
+void MultiDocumentPanel::closeDocumentAsync (Component* component,
+                                             const bool checkItsOkToCloseFirst,
+                                             std::function<void (bool)> callback)
+{
+    // Intellisense warns about component being uninitialised.
+    // I'm not sure how a function argument could be uninitialised.
+    JUCE_BEGIN_IGNORE_WARNINGS_MSVC (6001)
+
+    if (component == nullptr)
+    {
+        if (callback != nullptr)
+            callback (true);
+
+        return;
+    }
+
+    if (components.contains (component))
+    {
+        if (checkItsOkToCloseFirst)
+        {
+            tryToCloseDocumentAsync (component,
+                                     [parent = SafePointer<MultiDocumentPanel> { this }, component, callback] (bool closedSuccessfully)
+            {
+                if (parent == nullptr)
+                    return;
+
+                if (closedSuccessfully)
+                    parent->closeDocumentInternal (component);
+
+                if (callback != nullptr)
+                    callback (closedSuccessfully);
+            });
+
+            return;
+        }
+
+        closeDocumentInternal (component);
+    }
+    else
+    {
+        jassertfalse;
+    }
+
+    if (callback != nullptr)
+        callback (true);
+
+    JUCE_END_IGNORE_WARNINGS_MSVC
 }
 
 int MultiDocumentPanel::getNumDocuments() const noexcept
