@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -100,14 +100,16 @@ public:
 
     /** Destructor.
 
-        When deleted, this will call closeAllDocuments (false) to make sure all its
+        When deleted, this will call close all open documents to make sure all its
         components are deleted. If you need to make sure all documents are saved
-        before closing, then you should call closeAllDocuments (true) and check that
-        it returns true before deleting the panel.
+        before closing, then you should call closeAllDocumentsAsync() with
+        checkItsOkToCloseFirst == true and check the provided callback result is true
+        before deleting the panel.
     */
     ~MultiDocumentPanel() override;
 
     //==============================================================================
+   #if JUCE_MODAL_LOOPS_PERMITTED
     /** Tries to close all the documents.
 
         If checkItsOkToCloseFirst is true, then the tryToCloseDocument() method will
@@ -120,6 +122,22 @@ public:
         @see closeDocument
     */
     bool closeAllDocuments (bool checkItsOkToCloseFirst);
+   #endif
+
+    /** Tries to close all the documents.
+
+        If checkItsOkToCloseFirst is true, then the tryToCloseDocumentAsync() method will
+        be called for each open document, and any of these calls fails, this method
+        will stop and provide an argument of false to the callback, leaving some documents
+        still open.
+
+        If checkItsOkToCloseFirst is false, then all documents will be closed
+        unconditionally.
+
+        @see closeDocumentAsync
+    */
+    void closeAllDocumentsAsync (bool checkItsOkToCloseFirst,
+                                 std::function<void (bool)> callback);
 
     /** Adds a document component to the panel.
 
@@ -134,14 +152,15 @@ public:
         @param component            the component to add
         @param backgroundColour     the background colour to use to fill the component's
                                     window or tab
-        @param deleteWhenRemoved    if true, then when the component is removed by closeDocument()
-                                    or closeAllDocuments(), then it will be deleted. If false, then
+        @param deleteWhenRemoved    if true, then when the component is removed by closeDocumentAsync()
+                                    or closeAllDocumentsAsync(), then it will be deleted. If false, then
                                     the caller must handle the component's deletion
     */
     bool addDocument (Component* component,
                       Colour backgroundColour,
                       bool deleteWhenRemoved);
 
+   #if JUCE_MODAL_LOOPS_PERMITTED
     /** Closes one of the documents.
 
         If checkItsOkToCloseFirst is true, then the tryToCloseDocument() method will
@@ -158,6 +177,25 @@ public:
     */
     bool closeDocument (Component* component,
                         bool checkItsOkToCloseFirst);
+   #endif
+
+    /** Closes one of the documents.
+
+        If checkItsOkToCloseFirst is true, then the tryToCloseDocumentAsync() method will
+        be called, and if it fails, this method will call the callback with a false
+        argument without closing the document.
+
+        If checkItsOkToCloseFirst is false, then the documents will be closed
+        unconditionally.
+
+        The component will be deleted if the deleteWhenRemoved parameter was set to
+        true when it was added with addDocument.
+
+        @see addDocument, closeAllDocumentsAsync
+    */
+    void closeDocumentAsync (Component* component,
+                             bool checkItsOkToCloseFirst,
+                             std::function<void (bool)> callback);
 
     /** Returns the number of open document windows.
 
@@ -248,6 +286,7 @@ public:
     TabbedComponent* getCurrentTabbedComponent() const noexcept         { return tabComponent.get(); }
 
     //==============================================================================
+   #if JUCE_MODAL_LOOPS_PERMITTED
     /** A subclass must override this to say whether its currently ok for a document
         to be closed.
 
@@ -269,7 +308,32 @@ public:
 
         @see closeDocument, FileBasedDocument::saveIfNeededAndUserAgrees()
     */
-    virtual bool tryToCloseDocument (Component* component) = 0;
+    virtual bool tryToCloseDocument (Component* component);
+   #endif
+
+    /** A subclass must override this to say whether its currently ok for a document
+        to be closed.
+
+        This method is called by closeDocumentAsync() and closeAllDocumentsAsync()
+        to indicate that a document should be saved if possible, ready for it to be closed.
+
+        If the callback is called with a true argument, then it means the document is ok
+        and can be closed.
+
+        If the callback is called with a false argument, then it means that the
+        closeDocumentAsync() method should stop and not close.
+
+        Normally, you'd use this method to ask the user if they want to save any changes,
+        then return true if the save operation went ok. If the user cancelled the save
+        operation you could give a value of false to the callback to abort the close operation.
+
+        If your component is based on the FileBasedDocument class, then you'd probably want
+        to call FileBasedDocument::saveIfNeededAndUserAgreesAsync() and call the callback with
+        true if this returned FileBasedDocument::savedOk.
+
+        @see closeDocumentAsync, FileBasedDocument::saveIfNeededAndUserAgreesAsync()
+    */
+    virtual void tryToCloseDocumentAsync (Component* component, std::function<void (bool)> callback) = 0;
 
     /** Creates a new window to be used for a document.
 
@@ -288,12 +352,12 @@ public:
 
 private:
     //==============================================================================
-    LayoutMode mode = MaximisedWindowsWithTabs;
-    Array<Component*> components;
-    std::unique_ptr<TabbedComponent> tabComponent;
-    Colour backgroundColour { Colours::lightblue };
-    int maximumNumDocuments = 0, numDocsBeforeTabsUsed = 0;
+    void closeDocumentInternal (Component*);
+    static void closeLastDocumentRecursive (SafePointer<MultiDocumentPanel>,
+                                            bool,
+                                            std::function<void (bool)>);
 
+    //==============================================================================
     struct TabbedComponentInternal;
     friend class MultiDocumentPanelWindow;
 
@@ -301,6 +365,12 @@ private:
     void updateOrder();
     void addWindow (Component*);
 
+    LayoutMode mode = MaximisedWindowsWithTabs;
+    Array<Component*> components;
+    std::unique_ptr<TabbedComponent> tabComponent;
+    Colour backgroundColour { Colours::lightblue };
+    int maximumNumDocuments = 0, numDocsBeforeTabsUsed = 0;
+
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (MultiDocumentPanel)
 };
 
