@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-7-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -35,6 +34,7 @@ LookAndFeel_V3::LookAndFeel_V3()
     setColour (TextButton::buttonColourId, textButtonColour);
     setColour (TextButton::buttonOnColourId, Colour (0xff888888));
     setColour (ComboBox::buttonColourId, textButtonColour);
+    setColour (ComboBox::focusedOutlineColourId, textButtonColour);
     setColour (TextEditor::outlineColourId, Colours::transparentBlack);
     setColour (TabbedButtonBar::tabOutlineColourId, Colour (0x66000000));
     setColour (TabbedComponent::outlineColourId, Colour (0x66000000));
@@ -64,15 +64,15 @@ void LookAndFeel_V3::drawScrollbar (Graphics& g, ScrollBar& scrollbar, int x, in
 
     if (thumbSize > 0)
     {
-        const float thumbIndent = (isScrollbarVertical ? width : height) * 0.25f;
+        const float thumbIndent = (float) (isScrollbarVertical ? width : height) * 0.25f;
         const float thumbIndentx2 = thumbIndent * 2.0f;
 
         if (isScrollbarVertical)
-            thumbPath.addRoundedRectangle (x + thumbIndent, thumbStartPosition + thumbIndent,
-                                           width - thumbIndentx2, thumbSize - thumbIndentx2, (width - thumbIndentx2) * 0.5f);
+            thumbPath.addRoundedRectangle ((float) x + thumbIndent, (float) thumbStartPosition + thumbIndent,
+                                           (float) width - thumbIndentx2, (float) thumbSize - thumbIndentx2, ((float) width - thumbIndentx2) * 0.5f);
         else
-            thumbPath.addRoundedRectangle (thumbStartPosition + thumbIndent, y + thumbIndent,
-                                           thumbSize - thumbIndentx2, height - thumbIndentx2, (height - thumbIndentx2) * 0.5f);
+            thumbPath.addRoundedRectangle ((float) thumbStartPosition + thumbIndent, (float) y + thumbIndent,
+                                           (float) thumbSize - thumbIndentx2, (float) height - thumbIndentx2, ((float) height - thumbIndentx2) * 0.5f);
     }
 
     Colour thumbCol (scrollbar.findColour (ScrollBar::thumbColourId, true));
@@ -93,8 +93,8 @@ void LookAndFeel_V3::drawConcertinaPanelHeader (Graphics& g, const Rectangle<int
 {
     const Colour bkg (Colours::grey);
 
-    g.setGradientFill (ColourGradient (Colours::white.withAlpha (isMouseOver ? 0.4f : 0.2f), 0, (float) area.getY(),
-                                       Colours::darkgrey.withAlpha (0.1f), 0, (float) area.getBottom(), false));
+    g.setGradientFill (ColourGradient::vertical (Colours::white.withAlpha (isMouseOver ? 0.4f : 0.2f), (float) area.getY(),
+                                                 Colours::darkgrey.withAlpha (0.1f), (float) area.getBottom()));
     g.fillAll();
 
     g.setColour (bkg.contrasting().withAlpha (0.1f));
@@ -102,7 +102,7 @@ void LookAndFeel_V3::drawConcertinaPanelHeader (Graphics& g, const Rectangle<int
     g.fillRect (area.withTop (area.getBottom() - 1));
 
     g.setColour (bkg.contrasting());
-    g.setFont (Font (area.getHeight() * 0.6f).boldened());
+    g.setFont (Font ((float) area.getHeight() * 0.6f).boldened());
     g.drawFittedText (panel.getName(), 4, 0, area.getWidth() - 6, area.getHeight(), Justification::centredLeft, 1);
 }
 
@@ -111,8 +111,8 @@ static void drawButtonShape (Graphics& g, const Path& outline, Colour baseColour
     const float mainBrightness = baseColour.getBrightness();
     const float mainAlpha = baseColour.getFloatAlpha();
 
-    g.setGradientFill (ColourGradient (baseColour.brighter (0.2f), 0.0f, 0.0f,
-                                       baseColour.darker (0.25f), 0.0f, height, false));
+    g.setGradientFill (ColourGradient::vertical (baseColour.brighter (0.2f), 0.0f,
+                                                 baseColour.darker (0.25f), height));
     g.fillPath (outline);
 
     g.setColour (Colours::white.withAlpha (0.4f * mainAlpha * mainBrightness * mainBrightness));
@@ -124,21 +124,21 @@ static void drawButtonShape (Graphics& g, const Path& outline, Colour baseColour
 }
 
 void LookAndFeel_V3::drawButtonBackground (Graphics& g, Button& button, const Colour& backgroundColour,
-                                           bool isMouseOverButton, bool isButtonDown)
+                                           bool shouldDrawButtonAsHighlighted, bool shouldDrawButtonAsDown)
 {
     Colour baseColour (backgroundColour.withMultipliedSaturation (button.hasKeyboardFocus (true) ? 1.3f : 0.9f)
                                        .withMultipliedAlpha (button.isEnabled() ? 0.9f : 0.5f));
 
-    if (isButtonDown || isMouseOverButton)
-        baseColour = baseColour.contrasting (isButtonDown ? 0.2f : 0.1f);
+    if (shouldDrawButtonAsDown || shouldDrawButtonAsHighlighted)
+        baseColour = baseColour.contrasting (shouldDrawButtonAsDown ? 0.2f : 0.1f);
 
     const bool flatOnLeft   = button.isConnectedOnLeft();
     const bool flatOnRight  = button.isConnectedOnRight();
     const bool flatOnTop    = button.isConnectedOnTop();
     const bool flatOnBottom = button.isConnectedOnBottom();
 
-    const float width  = button.getWidth() - 1.0f;
-    const float height = button.getHeight() - 1.0f;
+    const float width  = (float) button.getWidth()  - 1.0f;
+    const float height = (float) button.getHeight() - 1.0f;
 
     if (width > 0 && height > 0)
     {
@@ -157,7 +157,7 @@ void LookAndFeel_V3::drawButtonBackground (Graphics& g, Button& button, const Co
 
 void LookAndFeel_V3::drawTableHeaderBackground (Graphics& g, TableHeaderComponent& header)
 {
-    Rectangle<int> r (header.getLocalBounds());
+    auto r = header.getLocalBounds();
     auto outlineColour = header.findColour (TableHeaderComponent::outlineColourId);
 
     g.setColour (outlineColour);
@@ -213,8 +213,8 @@ void LookAndFeel_V3::drawTabButton (TabBarButton& button, Graphics& g, bool isMo
             default:                              jassertfalse; break;
         }
 
-        g.setGradientFill (ColourGradient (bkg.brighter (0.2f), (float) p1.x, (float) p1.y,
-                                           bkg.darker (0.1f),   (float) p2.x, (float) p2.y, false));
+        g.setGradientFill (ColourGradient (bkg.brighter (0.2f), p1.toFloat(),
+                                           bkg.darker (0.1f),   p2.toFloat(), false));
     }
 
     g.fillRect (activeArea);
@@ -258,8 +258,8 @@ void LookAndFeel_V3::drawTabButton (TabBarButton& button, Graphics& g, bool isMo
 
     switch (o)
     {
-        case TabbedButtonBar::TabsAtLeft:   t = t.rotated (float_Pi * -0.5f).translated (area.getX(), area.getBottom()); break;
-        case TabbedButtonBar::TabsAtRight:  t = t.rotated (float_Pi *  0.5f).translated (area.getRight(), area.getY()); break;
+        case TabbedButtonBar::TabsAtLeft:   t = t.rotated (MathConstants<float>::pi * -0.5f).translated (area.getX(), area.getBottom()); break;
+        case TabbedButtonBar::TabsAtRight:  t = t.rotated (MathConstants<float>::pi *  0.5f).translated (area.getRight(), area.getY()); break;
         case TabbedButtonBar::TabsAtTop:
         case TabbedButtonBar::TabsAtBottom: t = t.translated (area.getX(), area.getY()); break;
         default:                            jassertfalse; break;
@@ -281,26 +281,26 @@ void LookAndFeel_V3::drawTabAreaBehindFrontButton (TabbedButtonBar& bar, Graphic
     {
         case TabbedButtonBar::TabsAtLeft:
             gradient.point1.x = (float) w;
-            gradient.point2.x = w * (1.0f - shadowSize);
+            gradient.point2.x = (float) w * (1.0f - shadowSize);
             shadowRect.setBounds ((int) gradient.point2.x, 0, w - (int) gradient.point2.x, h);
             line.setBounds (w - 1, 0, 1, h);
             break;
 
         case TabbedButtonBar::TabsAtRight:
-            gradient.point2.x = w * shadowSize;
+            gradient.point2.x = (float) w * shadowSize;
             shadowRect.setBounds (0, 0, (int) gradient.point2.x, h);
             line.setBounds (0, 0, 1, h);
             break;
 
         case TabbedButtonBar::TabsAtTop:
             gradient.point1.y = (float) h;
-            gradient.point2.y = h * (1.0f - shadowSize);
+            gradient.point2.y = (float) h * (1.0f - shadowSize);
             shadowRect.setBounds (0, (int) gradient.point2.y, w, h - (int) gradient.point2.y);
             line.setBounds (0, h - 1, w, 1);
             break;
 
         case TabbedButtonBar::TabsAtBottom:
-            gradient.point2.y = h * shadowSize;
+            gradient.point2.y = (float) h * shadowSize;
             shadowRect.setBounds (0, 0, w, (int) gradient.point2.y);
             line.setBounds (0, 0, w, 1);
             break;
@@ -352,16 +352,14 @@ int LookAndFeel_V3::getTreeViewIndentSize (TreeView&)
     return 20;
 }
 
-void LookAndFeel_V3::drawComboBox (Graphics& g, int width, int height, const bool /*isButtonDown*/,
+void LookAndFeel_V3::drawComboBox (Graphics& g, int width, int height, const bool /*isMouseButtonDown*/,
                                    int buttonX, int buttonY, int buttonW, int buttonH, ComboBox& box)
 {
     g.fillAll (box.findColour (ComboBox::backgroundColourId));
 
-    const Colour buttonColour (box.findColour (ComboBox::buttonColourId));
-
     if (box.isEnabled() && box.hasKeyboardFocus (false))
     {
-        g.setColour (buttonColour);
+        g.setColour (box.findColour (ComboBox::focusedOutlineColourId));
         g.drawRect (0, 0, width, height, 2);
     }
     else
@@ -373,14 +371,19 @@ void LookAndFeel_V3::drawComboBox (Graphics& g, int width, int height, const boo
     const float arrowX = 0.3f;
     const float arrowH = 0.2f;
 
-    Path p;
-    p.addTriangle (buttonX + buttonW * 0.5f,            buttonY + buttonH * (0.45f - arrowH),
-                   buttonX + buttonW * (1.0f - arrowX), buttonY + buttonH * 0.45f,
-                   buttonX + buttonW * arrowX,          buttonY + buttonH * 0.45f);
+    const auto x = (float) buttonX;
+    const auto y = (float) buttonY;
+    const auto w = (float) buttonW;
+    const auto h = (float) buttonH;
 
-    p.addTriangle (buttonX + buttonW * 0.5f,            buttonY + buttonH * (0.55f + arrowH),
-                   buttonX + buttonW * (1.0f - arrowX), buttonY + buttonH * 0.55f,
-                   buttonX + buttonW * arrowX,          buttonY + buttonH * 0.55f);
+    Path p;
+    p.addTriangle (x + w * 0.5f,            y + h * (0.45f - arrowH),
+                   x + w * (1.0f - arrowX), y + h * 0.45f,
+                   x + w * arrowX,          y + h * 0.45f);
+
+    p.addTriangle (x + w * 0.5f,            y + h * (0.55f + arrowH),
+                   x + w * (1.0f - arrowX), y + h * 0.55f,
+                   x + w * arrowX,          y + h * 0.55f);
 
     g.setColour (box.findColour (ComboBox::arrowColourId).withMultipliedAlpha (box.isEnabled() ? 1.0f : 0.3f));
     g.fillPath (p);
@@ -403,12 +406,12 @@ void LookAndFeel_V3::drawLinearSlider (Graphics& g, int x, int y, int width, int
         else
             p.addRectangle (fx, fy, sliderPos - fx, fh);
 
-        Colour baseColour (slider.findColour (Slider::thumbColourId)
+        auto baseColour = slider.findColour (Slider::thumbColourId)
                                 .withMultipliedSaturation (slider.isEnabled() ? 1.0f : 0.5f)
-                                .withMultipliedAlpha (0.8f));
+                                .withMultipliedAlpha (0.8f);
 
-        g.setGradientFill (ColourGradient (baseColour.brighter (0.08f), 0.0f, 0.0f,
-                                           baseColour.darker (0.08f), 0.0f, (float) height, false));
+        g.setGradientFill (ColourGradient::vertical (baseColour.brighter (0.08f), 0.0f,
+                                                     baseColour.darker (0.08f), (float) height));
         g.fillPath (p);
 
         g.setColour (baseColour.darker (0.2f));
@@ -440,21 +443,19 @@ void LookAndFeel_V3::drawLinearSliderBackground (Graphics& g, int x, int y, int
 
     if (slider.isHorizontal())
     {
-        const float iy = y + height * 0.5f - sliderRadius * 0.5f;
+        auto iy = (float) y + (float) height * 0.5f - sliderRadius * 0.5f;
 
-        g.setGradientFill (ColourGradient (gradCol1, 0.0f, iy,
-                                           gradCol2, 0.0f, iy + sliderRadius, false));
+        g.setGradientFill (ColourGradient::vertical (gradCol1, iy, gradCol2, iy + sliderRadius));
 
-        indent.addRoundedRectangle (x - sliderRadius * 0.5f, iy, width + sliderRadius, sliderRadius, 5.0f);
+        indent.addRoundedRectangle ((float) x - sliderRadius * 0.5f, iy, (float) width + sliderRadius, sliderRadius, 5.0f);
     }
     else
     {
-        const float ix = x + width * 0.5f - sliderRadius * 0.5f;
+        auto ix = (float) x + (float) width * 0.5f - sliderRadius * 0.5f;
 
-        g.setGradientFill (ColourGradient (gradCol1, ix, 0.0f,
-                                           gradCol2, ix + sliderRadius, 0.0f, false));
+        g.setGradientFill (ColourGradient::horizontal (gradCol1, ix, gradCol2, ix + sliderRadius));
 
-        indent.addRoundedRectangle (ix, y - sliderRadius * 0.5f, sliderRadius, height + sliderRadius, 5.0f);
+        indent.addRoundedRectangle (ix, (float) y - sliderRadius * 0.5f, sliderRadius, (float) height + sliderRadius, 5.0f);
     }
 
     g.fillPath (indent);
@@ -477,7 +478,7 @@ void LookAndFeel_V3::drawPopupMenuBackground (Graphics& g, int width, int height
 void LookAndFeel_V3::drawMenuBarBackground (Graphics& g, int width, int height,
                                             bool, MenuBarComponent& menuBar)
 {
-    const Colour colour (menuBar.findColour (PopupMenu::backgroundColourId));
+    auto colour = menuBar.findColour (PopupMenu::backgroundColourId);
 
     Rectangle<int> r (width, height);
 
@@ -485,7 +486,7 @@ void LookAndFeel_V3::drawMenuBarBackground (Graphics& g, int width, int height,
     g.fillRect (r.removeFromTop (1));
     g.fillRect (r.removeFromBottom (1));
 
-    g.setGradientFill (ColourGradient (colour, 0, 0, colour.darker (0.08f), 0, (float) height, false));
+    g.setGradientFill (ColourGradient::vertical (colour, 0, colour.darker (0.08f), (float) height));
     g.fillRect (r);
 }
 
@@ -504,7 +505,7 @@ void LookAndFeel_V3::drawKeymapChangeButton (Graphics& g, int width, int height,
         }
 
         g.setColour (textColour);
-        g.setFont (height * 0.6f);
+        g.setFont ((float) height * 0.6f);
         g.drawFittedText (keyDescription, 4, 0, width - 8, height, Justification::centred, 1);
     }
     else
@@ -520,7 +521,7 @@ void LookAndFeel_V3::drawKeymapChangeButton (Graphics& g, int width, int height,
         p.setUsingNonZeroWinding (false);
 
         g.setColour (textColour.darker(0.1f).withAlpha (button.isDown() ? 0.7f : (button.isOver() ? 0.5f : 0.3f)));
-        g.fillPath (p, p.getTransformToScaleToFit (2.0f, 2.0f, width - 4.0f, height - 4.0f, true));
+        g.fillPath (p, p.getTransformToScaleToFit (2.0f, 2.0f, (float) width - 4.0f, (float) height - 4.0f, true));
     }
 
     if (button.hasKeyboardFocus (false))
@@ -539,15 +540,15 @@ public:
     {
     }
 
-    void paintButton (Graphics& g, bool isMouseOverButton, bool isButtonDown) override
+    void paintButton (Graphics& g, bool shouldDrawButtonAsHighlighted, bool shouldDrawButtonAsDown) override
     {
         Colour background (Colours::grey);
 
         if (ResizableWindow* rw = findParentComponentOfClass<ResizableWindow>())
             background = rw->getBackgroundColour();
 
-        const float cx = getWidth() * 0.5f, cy = getHeight() * 0.5f;
-        const float diam = jmin (cx, cy) * (isButtonDown ? 0.60f : 0.65f);
+        const float cx = (float) getWidth() * 0.5f, cy = (float) getHeight() * 0.5f;
+        const float diam = jmin (cx, cy) * (shouldDrawButtonAsDown ? 0.60f : 0.65f);
 
         g.setColour (background);
         g.fillEllipse (cx - diam, cy - diam, diam * 2.0f, diam * 2.0f);
@@ -556,7 +557,7 @@ public:
 
         if (! isEnabled())
             c = c.withAlpha (0.6f);
-        else if (isMouseOverButton)
+        else if (shouldDrawButtonAsHighlighted)
             c = c.brighter();
 
         g.setColour (c);
