@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -156,6 +156,10 @@ public:
                                  const Rectangle<float>& body) = 0;
     };
 
+    //==============================================================================
+    /** @internal */
+    void paint (Graphics&) override;
+
 protected:
     //==============================================================================
     /** Subclasses should override this to return the size of the content they
@@ -170,10 +174,6 @@ protected:
     */
     virtual void paintContent (Graphics& g, int width, int height) = 0;
 
-public:
-    /** @internal */
-    void paint (Graphics&) override;
-
 private:
     Rectangle<int> content;
     Point<int> arrowTip;
