@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -35,16 +35,17 @@ class DragAndDropContainer::DragImageComponent  : public Component,
                                                   private Timer
 {
 public:
-    DragImageComponent (const Image& im,
+    DragImageComponent (const ScaledImage& im,
                         const var& desc,
                         Component* const sourceComponent,
                         const MouseInputSource* draggingSource,
                         DragAndDropContainer& ddc,
                         Point<int> offset)
         : sourceDetails (desc, sourceComponent, Point<int>()),
-          image (im), owner (ddc),
+          image (im),
+          owner (ddc),
           mouseDragSource (draggingSource->getComponentUnderMouse()),
-          imageOffset (offset),
+          imageOffset (transformOffsetCoordinates (sourceComponent, offset)),
           originalInputSourceIndex (draggingSource->getIndex()),
           originalInputSourceType (draggingSource->getType())
     {
@@ -58,6 +59,7 @@ public:
         startTimer (200);
 
         setInterceptsMouseClicks (false, false);
+        setWantsKeyboardFocus (true);
         setAlwaysOnTop (true);
     }
 
@@ -83,7 +85,7 @@ public:
             g.fillAll (Colours::white);
 
         g.setOpacity (1.0f);
-        g.drawImageAt (image, 0, 0);
+        g.drawImage (image.getImage(), getLocalBounds().toFloat());
     }
 
     void mouseUp (const MouseEvent& e) override
@@ -164,7 +166,7 @@ public:
         forceMouseCursorUpdate();
     }
 
-    void updateImage (const Image& newImage)
+    void updateImage (const ScaledImage& newImage)
     {
         image = newImage;
         updateSize();
@@ -199,7 +201,12 @@ public:
     {
         if (key == KeyPress::escapeKey)
         {
-            dismissWithAnimation (true);
+            const auto wasVisible = isVisible();
+            setVisible (false);
+
+            if (wasVisible)
+                dismissWithAnimation (true);
+
             deleteSelf();
             return true;
         }
@@ -218,7 +225,7 @@ public:
     DragAndDropTarget::SourceDetails sourceDetails;
 
 private:
-    Image image;
+    ScaledImage image;
     DragAndDropContainer& owner;
     WeakReference<Component> mouseDragSource, currentlyOverComp;
     const Point<int> imageOffset;
@@ -229,7 +236,8 @@ private:
 
     void updateSize()
     {
-        setSize (image.getWidth(), image.getHeight());
+        const auto bounds = image.getScaledBounds().toNearestInt();
+        setSize (bounds.getWidth(), bounds.getHeight());
     }
 
     void forceMouseCursorUpdate()
@@ -263,6 +271,11 @@ private:
         return nullptr;
     }
 
+    Point<int> transformOffsetCoordinates (const Component* const sourceComponent, Point<int> offsetInSource) const
+    {
+        return getLocalPoint (sourceComponent, offsetInSource) - getLocalPoint (sourceComponent, Point<int>());
+    }
+
     DragAndDropTarget* findTarget (Point<int> screenPos, Point<int>& relativePos,
                                    Component*& resultComponent) const
     {
@@ -383,17 +396,13 @@ private:
 
 
 //==============================================================================
-DragAndDropContainer::DragAndDropContainer()
-{
-}
+DragAndDropContainer::DragAndDropContainer() = default;
 
-DragAndDropContainer::~DragAndDropContainer()
-{
-}
+DragAndDropContainer::~DragAndDropContainer() = default;
 
 void DragAndDropContainer::startDragging (const var& sourceDescription,
                                           Component* sourceComponent,
-                                          Image dragImage,
+                                          const ScaledImage& dragImage,
                                           const bool allowDraggingToExternalWindows,
                                           const Point<int>* imageOffsetFromMouse,
                                           const MouseInputSource* inputSourceCausingDrag)
@@ -409,55 +418,53 @@ void DragAndDropContainer::startDragging (const var& sourceDescription,
         return;
     }
 
-    auto lastMouseDown = draggingSource->getLastMouseDownPosition().roundToInt();
-    Point<int> imageOffset;
+    const auto lastMouseDown = draggingSource->getLastMouseDownPosition().roundToInt();
 
-    if (dragImage.isNull())
+    struct ImageAndOffset
     {
-        dragImage = sourceComponent->createComponentSnapshot (sourceComponent->getLocalBounds())
-                       .convertedToFormat (Image::ARGB);
+        ScaledImage image;
+        Point<double> offset;
+    };
 
-        dragImage.multiplyAllAlphas (0.6f);
-
-        auto lo = 150;
-        auto hi = 400;
-
-        auto relPos = sourceComponent->getLocalPoint (nullptr, lastMouseDown);
-        auto clipped = dragImage.getBounds().getConstrainedPoint (relPos);
-        Random random;
-
-        for (auto y = dragImage.getHeight(); --y >= 0;)
-        {
-            auto dy = (y - clipped.getY()) * (y - clipped.getY());
-
-            for (auto x = dragImage.getWidth(); --x >= 0;)
-            {
-                auto dx = x - clipped.getX();
-                auto distance = roundToInt (std::sqrt (dx * dx + dy));
-
-                if (distance > lo)
-                {
-                    auto alpha = (distance > hi) ? 0
-                                                 : (float) (hi - distance) / (float) (hi - lo)
-                                                     + random.nextFloat() * 0.008f;
-
-                    dragImage.multiplyAlphaAt (x, y, alpha);
-                }
-            }
-        }
-
-        imageOffset = clipped;
-    }
-    else
+    const auto imageToUse = [&]() -> ImageAndOffset
     {
-        if (imageOffsetFromMouse == nullptr)
-            imageOffset = dragImage.getBounds().getCentre();
-        else
-            imageOffset = dragImage.getBounds().getConstrainedPoint (-*imageOffsetFromMouse);
-    }
+        if (! dragImage.getImage().isNull())
+            return { dragImage, imageOffsetFromMouse != nullptr ? dragImage.getScaledBounds().getConstrainedPoint (-imageOffsetFromMouse->toDouble())
+                                                                : dragImage.getScaledBounds().getCentre() };
 
-    auto* dragImageComponent = dragImageComponents.add (new DragImageComponent (dragImage, sourceDescription, sourceComponent,
-                                                                                draggingSource, *this, imageOffset));
+        const auto scaleFactor = 2.0;
+        auto image = sourceComponent->createComponentSnapshot (sourceComponent->getLocalBounds(), true, (float) scaleFactor)
+                                    .convertedToFormat (Image::ARGB);
+        image.multiplyAllAlphas (0.6f);
+
+        const auto relPos = sourceComponent->getLocalPoint (nullptr, lastMouseDown).toDouble();
+        const auto clipped = (image.getBounds().toDouble() / scaleFactor).getConstrainedPoint (relPos);
+
+        Image fade (Image::SingleChannel, image.getWidth(), image.getHeight(), true);
+        Graphics fadeContext (fade);
+
+        ColourGradient gradient;
+        gradient.isRadial = true;
+        gradient.point1 = clipped.toFloat() * scaleFactor;
+        gradient.point2 = gradient.point1 + Point<float> (0.0f, scaleFactor * 400.0f);
+        gradient.addColour (0.0, Colours::white);
+        gradient.addColour (0.375, Colours::white);
+        gradient.addColour (1.0, Colours::transparentWhite);
+
+        fadeContext.setGradientFill (gradient);
+        fadeContext.fillAll();
+
+        Image composite (Image::ARGB, image.getWidth(), image.getHeight(), true);
+        Graphics compositeContext (composite);
+
+        compositeContext.reduceClipRegion (fade, {});
+        compositeContext.drawImageAt (image, 0, 0);
+
+        return { ScaledImage (composite, scaleFactor), clipped };
+    }();
+
+    auto* dragImageComponent = dragImageComponents.add (new DragImageComponent (imageToUse.image, sourceDescription, sourceComponent,
+                                                                                draggingSource, *this, imageToUse.offset.roundToInt()));
 
     if (allowDraggingToExternalWindows)
     {
@@ -465,8 +472,7 @@ void DragAndDropContainer::startDragging (const var& sourceDescription,
             dragImageComponent->setOpaque (true);
 
         dragImageComponent->addToDesktop (ComponentPeer::windowIgnoresMouseClicks
-                                          | ComponentPeer::windowIsTemporary
-                                          | ComponentPeer::windowIgnoresKeyPresses);
+                                          | ComponentPeer::windowIsTemporary);
     }
     else
     {
@@ -483,6 +489,7 @@ void DragAndDropContainer::startDragging (const var& sourceDescription,
 
     dragImageComponent->sourceDetails.localPosition = sourceComponent->getLocalPoint (nullptr, lastMouseDown);
     dragImageComponent->updateLocation (false, lastMouseDown);
+    dragImageComponent->grabKeyboardFocus();
 
    #if JUCE_WINDOWS
     // Under heavy load, the layered window's paint callback can often be lost by the OS,
@@ -522,7 +529,7 @@ var DragAndDropContainer::getDragDescriptionForIndex (int index) const
     return dragImageComponents.getUnchecked (index)->sourceDetails.description;
 }
 
-void DragAndDropContainer::setCurrentDragImage (const Image& newImage)
+void DragAndDropContainer::setCurrentDragImage (const ScaledImage& newImage)
 {
     // If you are performing drag and drop in a multi-touch environment then
     // you should use the setDragImageForIndex() method instead!
@@ -531,7 +538,7 @@ void DragAndDropContainer::setCurrentDragImage (const Image& newImage)
     dragImageComponents[0]->updateImage (newImage);
 }
 
-void DragAndDropContainer::setDragImageForIndex (int index, const Image& newImage)
+void DragAndDropContainer::setDragImageForIndex (int index, const ScaledImage& newImage)
 {
     if (isPositiveAndBelow (index, dragImageComponents.size()))
         dragImageComponents.getUnchecked (index)->updateImage (newImage);
