@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -26,32 +26,25 @@
 namespace juce
 {
 
-// Win32NativeFileChooser needs to be a reference counted object as there
-// is no way for the parent to know when the dialog HWND has actually been
-// created without pumping the message thread (which is forbidden when modal
-// loops are disabled). However, the HWND pointer is the only way to cancel
-// the dialog box. This means that the actual native FileChooser HWND may
-// not have been created yet when the user deletes JUCE's FileChooser class. If this
-// occurs the Win32NativeFileChooser will still have a reference count of 1 and will
-// simply delete itself immediately once the HWND will have been created a while later.
-class Win32NativeFileChooser  : public ReferenceCountedObject,
-                                private Thread
+// Implemented in juce_win32_Messaging.cpp
+bool dispatchNextMessageOnSystemQueue (bool returnIfNoPendingMessages);
+
+class Win32NativeFileChooser  : private Thread
 {
 public:
-    using Ptr = ReferenceCountedObjectPtr<Win32NativeFileChooser>;
-
     enum { charsAvailableForResult = 32768 };
 
     Win32NativeFileChooser (Component* parent, int flags, FilePreviewComponent* previewComp,
                             const File& startingFile, const String& titleToUse,
                             const String& filtersToUse)
         : Thread ("Native Win32 FileChooser"),
-          owner (parent), title (titleToUse), filtersString (filtersToUse),
+          owner (parent),
+          title (titleToUse),
+          filtersString (filtersToUse.replaceCharacter (',', ';')),
           selectsDirectories ((flags & FileBrowserComponent::canSelectDirectories)   != 0),
           isSave             ((flags & FileBrowserComponent::saveMode)               != 0),
           warnAboutOverwrite ((flags & FileBrowserComponent::warnAboutOverwriting)   != 0),
-          selectMultiple     ((flags & FileBrowserComponent::canSelectMultipleItems) != 0),
-          nativeDialogRef (nullptr), shouldCancel (0)
+          selectMultiple     ((flags & FileBrowserComponent::canSelectMultipleItems) != 0)
     {
         auto parentDirectory = startingFile.getParentDirectory();
 
@@ -78,10 +71,15 @@ public:
         }
     }
 
-    ~Win32NativeFileChooser()
+    ~Win32NativeFileChooser() override
     {
         signalThreadShouldExit();
-        waitForThreadToExit (-1);
+
+        while (isThreadRunning())
+        {
+            if (! dispatchNextMessageOnSystemQueue (true))
+                Thread::sleep (1);
+        }
     }
 
     void open (bool async)
@@ -95,10 +93,7 @@ public:
         {
             jassert (! isThreadRunning());
 
-            threadHasReference.reset();
             startThread();
-
-            threadHasReference.wait (-1);
         }
         else
         {
@@ -112,10 +107,10 @@ public:
         ScopedLock lock (deletingDialog);
 
         customComponent = nullptr;
-        shouldCancel.set (1);
+        shouldCancel = true;
 
         if (auto hwnd = nativeDialogRef.get())
-            EndDialog (hwnd, 0);
+            PostMessage (hwnd, WM_CLOSE, 0, 0);
     }
 
     Component* getCustomComponent()    { return customComponent.get(); }
@@ -151,12 +146,11 @@ private:
     };
 
     //==============================================================================
-    Component::SafePointer<Component> owner;
+    const Component::SafePointer<Component> owner;
     String title, filtersString;
     std::unique_ptr<CustomComponentHolder> customComponent;
     String initialPath, returnedString;
 
-    WaitableEvent threadHasReference;
     CriticalSection deletingDialog;
 
     bool selectsDirectories, isSave, warnAboutOverwrite, selectMultiple;
@@ -164,11 +158,16 @@ private:
     HeapBlock<WCHAR> files;
     HeapBlock<WCHAR> filters;
 
-    Atomic<HWND> nativeDialogRef;
-    Atomic<int>  shouldCancel;
+    Atomic<HWND> nativeDialogRef { nullptr };
+    bool shouldCancel = false;
+
+    struct FreeLPWSTR
+    {
+        void operator() (LPWSTR ptr) const noexcept { CoTaskMemFree (ptr); }
+    };
 
    #if JUCE_MSVC
-    bool showDialog (IFileDialog& dialog, bool async) const
+    bool showDialog (IFileDialog& dialog, bool async)
     {
         FILEOPENDIALOGOPTIONS flags = {};
 
@@ -194,7 +193,17 @@ private:
         PIDLIST_ABSOLUTE pidl = {};
 
         if (FAILED (SHParseDisplayName (initialPath.toWideCharPointer(), nullptr, &pidl, SFGAO_FOLDER, nullptr)))
-            return false;
+        {
+            LPWSTR ptr = nullptr;
+            auto result = SHGetKnownFolderPath (FOLDERID_Desktop, 0, nullptr, &ptr);
+            std::unique_ptr<WCHAR, FreeLPWSTR> desktopPath (ptr);
+
+            if (FAILED (result))
+                return false;
+
+            if (FAILED (SHParseDisplayName (desktopPath.get(), nullptr, &pidl, SFGAO_FOLDER, nullptr)))
+                return false;
+        }
 
         const auto item = [&]
         {
@@ -203,8 +212,13 @@ private:
             return ptr;
         }();
 
-        if (item == nullptr || FAILED (dialog.SetFolder (item)))
-            return false;
+        if (item != nullptr)
+        {
+            dialog.SetDefaultFolder (item);
+
+            if (! initialPath.isEmpty())
+                dialog.SetFolder (item);
+        }
 
         String filename (files.getData());
 
@@ -221,7 +235,64 @@ private:
         if (! selectsDirectories && FAILED (dialog.SetFileTypes (numElementsInArray (spec), spec)))
             return false;
 
-        return dialog.Show (static_cast<HWND> (async ? nullptr : owner->getWindowHandle())) == S_OK;
+        struct Events  : public ComBaseClassHelper<IFileDialogEvents>
+        {
+            explicit Events (Win32NativeFileChooser& o) : owner (o) {}
+
+            JUCE_COMRESULT OnTypeChange (IFileDialog* d) override                                                 { return updateHwnd (d); }
+            JUCE_COMRESULT OnFolderChanging (IFileDialog* d, IShellItem*) override                                { return updateHwnd (d); }
+            JUCE_COMRESULT OnFileOk (IFileDialog* d) override                                                     { return updateHwnd (d); }
+            JUCE_COMRESULT OnFolderChange (IFileDialog* d) override                                               { return updateHwnd (d); }
+            JUCE_COMRESULT OnSelectionChange (IFileDialog* d) override                                            { return updateHwnd (d); }
+            JUCE_COMRESULT OnShareViolation (IFileDialog* d, IShellItem*, FDE_SHAREVIOLATION_RESPONSE*) override  { return updateHwnd (d); }
+            JUCE_COMRESULT OnOverwrite (IFileDialog* d, IShellItem*, FDE_OVERWRITE_RESPONSE*) override            { return updateHwnd (d); }
+
+            JUCE_COMRESULT updateHwnd (IFileDialog* d)
+            {
+                HWND hwnd = nullptr;
+
+                if (auto window = ComSmartPtr<IFileDialog> { d }.getInterface<IOleWindow>())
+                    window->GetWindow (&hwnd);
+
+                ScopedLock lock (owner.deletingDialog);
+
+                if (owner.shouldCancel)
+                    d->Close (S_FALSE);
+                else if (hwnd != nullptr)
+                    owner.nativeDialogRef = hwnd;
+
+                return S_OK;
+            }
+
+            Win32NativeFileChooser& owner;
+        };
+
+        {
+            ScopedLock lock (deletingDialog);
+
+            if (shouldCancel)
+                return false;
+        }
+
+        const auto result = [&]
+        {
+            struct ScopedAdvise
+            {
+                ScopedAdvise (IFileDialog& d, Events& events) : dialog (d) { dialog.Advise (&events, &cookie); }
+                ~ScopedAdvise() { dialog.Unadvise (cookie); }
+                IFileDialog& dialog;
+                DWORD cookie = 0;
+            };
+
+            Events events { *this };
+            ScopedAdvise scope { dialog, events };
+            return dialog.Show (async ? nullptr : static_cast<HWND> (owner->getWindowHandle())) == S_OK;
+        }();
+
+        ScopedLock lock (deletingDialog);
+        nativeDialogRef = nullptr;
+
+        return result;
     }
 
     //==============================================================================
@@ -229,14 +300,13 @@ private:
     {
         const auto getUrl = [] (IShellItem& item)
         {
-            struct Free
-            {
-                void operator() (LPWSTR ptr) const noexcept { CoTaskMemFree (ptr); }
-            };
-
             LPWSTR ptr = nullptr;
-            item.GetDisplayName (SIGDN_URL, &ptr);
-            return std::unique_ptr<WCHAR, Free> { ptr };
+
+            if (item.GetDisplayName (SIGDN_FILESYSPATH, &ptr) != S_OK)
+                return URL();
+
+            const auto path = std::unique_ptr<WCHAR, FreeLPWSTR> { ptr };
+            return URL (File (String (path.get())));
         };
 
         if (isSave)
@@ -263,7 +333,12 @@ private:
             if (item == nullptr)
                 return {};
 
-            return { URL (String (getUrl (*item).get())) };
+            const auto url = getUrl (*item);
+
+            if (url.isEmpty())
+                return {};
+
+            return { url };
         }
 
         const auto dialog = [&]
@@ -299,7 +374,12 @@ private:
             items->GetItemAt (i, scope.resetAndGetPointerAddress());
 
             if (scope != nullptr)
-                result.add (String (getUrl (*scope).get()));
+            {
+                const auto url = getUrl (*scope);
+
+                if (! url.isEmpty())
+                    result.add (url);
+            }
         }
 
         return result;
@@ -356,6 +436,13 @@ private:
            #else
             of.lStructSize = sizeof (of);
            #endif
+
+            if (files[0] != 0)
+            {
+                auto startingFile = File (initialPath).getChildFile (String (files.get()));
+                startingFile.getFullPathName().copyToUTF16 (files, charsAvailableForResult * sizeof (WCHAR));
+            }
+
             of.hwndOwner = (HWND) (async ? nullptr : owner->getWindowHandle());
             of.lpstrFilter = filters.getData();
             of.nFilterIndex = 1;
@@ -415,8 +502,11 @@ private:
         const Remover remover (*this);
 
        #if JUCE_MSVC
-        if (SystemStats::getOperatingSystemType() >= SystemStats::WinVista)
+        if (SystemStats::getOperatingSystemType() >= SystemStats::WinVista
+            && customComponent == nullptr)
+        {
             return openDialogVistaAndUp (async);
+        }
        #endif
 
         return openDialogPreVista (async);
@@ -424,17 +514,27 @@ private:
 
     void run() override
     {
-        // as long as the thread is running, don't delete this class
-        Ptr safeThis (this);
-        threadHasReference.signal();
-
-        auto r = openDialog (true);
-        MessageManager::callAsync ([safeThis, r]
+        results = [&]
         {
-            safeThis->results = r;
+            struct ScopedCoInitialize
+            {
+                // IUnknown_GetWindow will only succeed when instantiated in a single-thread apartment
+                ScopedCoInitialize() { ignoreUnused (CoInitializeEx (nullptr, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE)); }
+                ~ScopedCoInitialize() { CoUninitialize(); }
+            };
 
-            if (safeThis->owner != nullptr)
-                safeThis->owner->exitModalState (r.size() > 0 ? 1 : 0);
+            ScopedCoInitialize scope;
+
+            return openDialog (true);
+        }();
+
+        auto safeOwner = owner;
+        auto resultCode = results.size() > 0 ? 1 : 0;
+
+        MessageManager::callAsync ([resultCode, safeOwner]
+        {
+            if (safeOwner != nullptr)
+                safeOwner->exitModalState (resultCode);
         });
     }
 
@@ -444,9 +544,9 @@ private:
         return dialogs;
     }
 
-    static Win32NativeFileChooser* getNativePointerForDialog (HWND hWnd)
+    static Win32NativeFileChooser* getNativePointerForDialog (HWND hwnd)
     {
-        return getNativeDialogList()[hWnd];
+        return getNativeDialogList()[hwnd];
     }
 
     //==============================================================================
@@ -514,7 +614,7 @@ private:
         ScopedLock lock (deletingDialog);
         getNativeDialogList().set (hdlg, this);
 
-        if (shouldCancel.get() != 0)
+        if (shouldCancel)
         {
             EndDialog (hdlg, 0);
         }
@@ -533,7 +633,7 @@ private:
                 auto screenRectangle = Rectangle<int>::leftTopRightBottom (dialogScreenRect.left,  dialogScreenRect.top,
                                                                            dialogScreenRect.right, dialogScreenRect.bottom);
 
-                auto scale = Desktop::getInstance().getDisplays().findDisplayForRect (screenRectangle, true).scale;
+                auto scale = Desktop::getInstance().getDisplays().getDisplayForRect (screenRectangle, true)->scale;
                 auto physicalComponentWidth = roundToInt (safeCustomComponent->getWidth() * scale);
 
                 SetWindowPos (hdlg, nullptr, screenRectangle.getX(), screenRectangle.getY(),
@@ -579,7 +679,7 @@ private:
     {
         ScopedLock lock (deletingDialog);
 
-        if (customComponent != nullptr && shouldCancel.get() == 0)
+        if (customComponent != nullptr && ! shouldCancel)
         {
             if (FilePreviewComponent* comp = dynamic_cast<FilePreviewComponent*> (customComponent->getChildComponent (0)))
             {
@@ -592,12 +692,11 @@ private:
                 }
                 else
                 {
-                    Component::SafePointer<FilePreviewComponent> safeComp (comp);
-
-                    File selectedFile (path);
-                    MessageManager::callAsync ([safeComp, selectedFile]() mutable
+                    MessageManager::callAsync ([safeComp = Component::SafePointer<FilePreviewComponent> { comp },
+                                                selectedFile = File { path }]() mutable
                                                {
-                                                    safeComp->selectedFileChanged (selectedFile);
+                                                    if (safeComp != nullptr)
+                                                        safeComp->selectedFileChanged (selectedFile);
                                                });
                 }
             }
@@ -677,16 +776,17 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Win32NativeFileChooser)
 };
 
-class FileChooser::Native     : public Component,
+class FileChooser::Native     : public std::enable_shared_from_this<Native>,
+                                public Component,
                                 public FileChooser::Pimpl
 {
 public:
-    Native (FileChooser& fileChooser, int flags, FilePreviewComponent* previewComp)
+    Native (FileChooser& fileChooser, int flagsIn, FilePreviewComponent* previewComp)
         : owner (fileChooser),
-          nativeFileChooser (new Win32NativeFileChooser (this, flags, previewComp, fileChooser.startingFile,
-                                                         fileChooser.title, fileChooser.filters))
+          nativeFileChooser (std::make_unique<Win32NativeFileChooser> (this, flagsIn, previewComp, fileChooser.startingFile,
+                                                                       fileChooser.title, fileChooser.filters))
     {
-        auto mainMon = Desktop::getInstance().getDisplays().getMainDisplay().userArea;
+        auto mainMon = Desktop::getInstance().getDisplays().getPrimaryDisplay()->userArea;
 
         setBounds (mainMon.getX() + mainMon.getWidth() / 4,
                    mainMon.getY() + mainMon.getHeight() / 4,
@@ -701,31 +801,33 @@ public:
     {
         exitModalState (0);
         nativeFileChooser->cancel();
-        nativeFileChooser = nullptr;
     }
 
     void launch() override
     {
-        SafePointer<Native> safeThis (this);
+        std::weak_ptr<Native> safeThis = shared_from_this();
 
-        enterModalState (true, ModalCallbackFunction::create (
-                         [safeThis] (int)
-                         {
-                             if (safeThis != nullptr)
-                                 safeThis->owner.finished (safeThis->nativeFileChooser->results);
-                         }));
+        enterModalState (true, ModalCallbackFunction::create ([safeThis] (int)
+        {
+            if (auto locked = safeThis.lock())
+                locked->owner.finished (locked->nativeFileChooser->results);
+        }));
 
         nativeFileChooser->open (true);
     }
 
     void runModally() override
     {
+       #if JUCE_MODAL_LOOPS_PERMITTED
         enterModalState (true);
         nativeFileChooser->open (false);
         exitModalState (nativeFileChooser->results.size() > 0 ? 1 : 0);
         nativeFileChooser->cancel();
 
         owner.finished (nativeFileChooser->results);
+       #else
+        jassertfalse;
+       #endif
     }
 
     bool canModalEventBeSentToComponent (const Component* targetComponent) override
@@ -741,7 +843,7 @@ public:
 
 private:
     FileChooser& owner;
-    Win32NativeFileChooser::Ptr nativeFileChooser;
+    std::shared_ptr<Win32NativeFileChooser> nativeFileChooser;
 };
 
 //==============================================================================
@@ -754,10 +856,10 @@ bool FileChooser::isPlatformDialogAvailable()
    #endif
 }
 
-FileChooser::Pimpl* FileChooser::showPlatformDialog (FileChooser& owner, int flags,
-                                                     FilePreviewComponent* preview)
+std::shared_ptr<FileChooser::Pimpl> FileChooser::showPlatformDialog (FileChooser& owner, int flags,
+                                                                     FilePreviewComponent* preview)
 {
-    return new FileChooser::Native (owner, flags, preview);
+    return std::make_shared<FileChooser::Native> (owner, flags, preview);
 }
 
 } // namespace juce
