@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -65,7 +65,8 @@ public:
         These properties are added without them being inside a named section. If you
         want them to be kept together in a collapsible section, use addSection() instead.
     */
-    void addProperties (const Array<PropertyComponent*>& newPropertyComponents);
+    void addProperties (const Array<PropertyComponent*>& newPropertyComponents,
+                        int extraPaddingBetweenComponents = 0);
 
     /** Adds a set of properties to the panel.
 
@@ -81,7 +82,8 @@ public:
     void addSection (const String& sectionTitle,
                      const Array<PropertyComponent*>& newPropertyComponents,
                      bool shouldSectionInitiallyBeOpen = true,
-                     int indexToInsertAt = -1);
+                     int indexToInsertAt = -1,
+                     int extraPaddingBetweenComponents = 0);
 
     /** Calls the refresh() method of all PropertyComponents in the panel */
     void refreshAll() const;
