@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -38,7 +38,7 @@ public:
 
     void paint (Graphics& g) override
     {
-        g.drawImageAt (image, 0, 0);
+        g.drawImage (image, getLocalBounds().toFloat());
     }
 
     Image image;
@@ -174,12 +174,13 @@ void TableHeaderComponent::setColumnWidth (const int columnId, const int newWidt
 {
     if (auto* ci = getInfoForId (columnId))
     {
-        if (ci->width != newWidth)
+        const auto newWidthToUse = jlimit (ci->minimumWidth, ci->maximumWidth, newWidth);
+
+        if (ci->width != newWidthToUse)
         {
             auto numColumns = getNumColumns (true);
 
-            ci->lastDeliberateWidth = ci->width
-                = jlimit (ci->minimumWidth, ci->maximumWidth, newWidth);
+            ci->lastDeliberateWidth = ci->width = newWidthToUse;
 
             if (stretchToFit)
             {
@@ -439,7 +440,7 @@ void TableHeaderComponent::restoreFromString (const String& storedVersion)
     {
         int index = 0;
 
-        forEachXmlChildElement (*storedXML, col)
+        for (auto* col : storedXML->getChildIterator())
         {
             auto tabId = col->getIntAttribute ("id");
 
@@ -693,7 +694,7 @@ void TableHeaderComponent::beginDrag (const MouseEvent& e)
             auto temp = columnIdBeingDragged;
             columnIdBeingDragged = 0;
 
-            dragOverlayComp.reset (new DragOverlayComp (createComponentSnapshot (columnRect, false)));
+            dragOverlayComp.reset (new DragOverlayComp (createComponentSnapshot (columnRect, false, 2.0f)));
             addAndMakeVisible (dragOverlayComp.get());
             columnIdBeingDragged = temp;
 
@@ -896,4 +897,10 @@ void TableHeaderComponent::Listener::tableColumnDraggingChanged (TableHeaderComp
 {
 }
 
+//==============================================================================
+std::unique_ptr<AccessibilityHandler> TableHeaderComponent::createAccessibilityHandler()
+{
+    return std::make_unique<AccessibilityHandler> (*this, AccessibilityRole::tableHeader);
+}
+
 } // namespace juce
