@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -68,7 +68,12 @@ public:
         See also the setReturnKeyStartsNewLine() method, which will also need to be turned
         on if you want a multi-line editor with line-breaks.
 
-        @see isMultiLine, setReturnKeyStartsNewLine
+        @param shouldBeMultiLine whether the editor should be multi- or single-line.
+        @param shouldWordWrap    sets whether long lines should be broken up in multi-line editors.
+                                 If this is false and scrollbars are enabled a horizontal scrollbar
+                                 will be shown.
+
+        @see isMultiLine, setReturnKeyStartsNewLine, setScrollbarsShown
     */
     void setMultiLine (bool shouldBeMultiLine,
                        bool shouldWordWrap = true);
@@ -140,13 +145,13 @@ public:
     bool isCaretVisible() const noexcept                            { return caretVisible && ! isReadOnly(); }
 
     //==============================================================================
-    /** Enables/disables a vertical scrollbar.
+    /** Enables or disables scrollbars (this only applies when in multi-line mode).
 
-        (This only applies when in multi-line mode). When the text gets too long to fit
-        in the component, a scrollbar can appear to allow it to be scrolled. Even when
-        this is enabled, the scrollbar will be hidden unless it's needed.
+        When the text gets too long to fit in the component, a scrollbar can appear to
+        allow it to be scrolled. Even when this is enabled, the scrollbar will be hidden
+        unless it's needed.
 
-        By default the scrollbar is enabled.
+        By default scrollbars are enabled.
     */
     void setScrollbarsShown (bool shouldBeEnabled);
 
@@ -460,7 +465,7 @@ public:
     /** Finds the index of the character at a given position.
         The coordinates are relative to the component's top-left.
     */
-    int getTextIndexAt (int x, int y);
+    int getTextIndexAt (int x, int y) const;
 
     /** Counts the number of characters in the text.
 
@@ -488,6 +493,16 @@ public:
     */
     void setIndents (int newLeftIndent, int newTopIndent);
 
+    /** Returns the gap at the top edge of the editor.
+        @see setIndents
+    */
+    int getTopIndent() const noexcept   { return topIndent; }
+
+    /** Returns the gap at the left edge of the editor.
+        @see setIndents
+    */
+    int getLeftIndent() const noexcept  { return leftIndent; }
+
     /** Changes the size of border left around the edge of the component.
         @see getBorder
     */
@@ -520,6 +535,14 @@ public:
     /** Returns the current line spacing of the TextEditor. */
     float getLineSpacing() const noexcept                           { return lineSpacing; }
 
+    /** Returns the bounding box for a range of text in the editor. As the range may span
+        multiple lines, this method returns a RectangleList.
+
+        The bounds are relative to the component's top-left and may extend beyond the bounds
+        of the component if the text is long and word wrapping is disabled.
+    */
+    RectangleList<int> getTextBounds (Range<int> textRange);
+
     //==============================================================================
     void moveCaretToEnd();
     bool moveCaretLeft (bool moveInWholeWordSteps, bool selecting);
@@ -642,8 +665,24 @@ public:
     void setInputRestrictions (int maxTextLength,
                                const String& allowedCharacters = String());
 
+    /** Sets the type of virtual keyboard that should be displayed when this editor has
+        focus.
+    */
     void setKeyboardType (VirtualKeyboardType type) noexcept    { keyboardType = type; }
 
+    /** Sets the behaviour of mouse/touch interactions outside this component.
+
+        If true, then presses outside of the TextEditor will dismiss the virtual keyboard.
+        If false, then the virtual keyboard will remain onscreen for as long as the TextEditor has
+        keyboard focus.
+    */
+    void setClicksOutsideDismissVirtualKeyboard (bool);
+
+    /** Returns true if the editor is configured to hide the virtual keyboard when the mouse is
+        pressed on another component.
+    */
+    bool getClicksOutsideDismissVirtualKeyboard() const     { return clicksOutsideDismissVirtualKeyboard; }
+
     //==============================================================================
     /** This abstract base class is implemented by LookAndFeel classes to provide
         TextEditor drawing functionality.
@@ -721,6 +760,7 @@ private:
     struct TextEditorViewport;
     struct InsertAction;
     struct RemoveAction;
+    class EditorAccessibilityHandler;
 
     std::unique_ptr<Viewport> viewport;
     TextHolderComponent* textHolder;
@@ -742,6 +782,8 @@ private:
     bool valueTextNeedsUpdating = false;
     bool consumeEscAndReturnKeys = true;
     bool underlineWhitespace = true;
+    bool mouseDownInEditor = false;
+    bool clicksOutsideDismissVirtualKeyboard = false;
 
     UndoManager undoManager;
     std::unique_ptr<CaretComponent> caret;
@@ -772,6 +814,7 @@ private:
     ListenerList<Listener> listeners;
     Array<Range<int>> underlinedSections;
 
+    std::unique_ptr<AccessibilityHandler> createAccessibilityHandler() override;
     void moveCaret (int newCaretPos);
     void moveCaretTo (int newPosition, bool isSelecting);
     void recreateCaret();
@@ -787,23 +830,23 @@ private:
     void updateCaretPosition();
     void updateValueFromText();
     void textWasChangedByValue();
-    int indexAtPosition (float x, float y);
+    int indexAtPosition (float x, float y) const;
     int findWordBreakAfter (int position) const;
     int findWordBreakBefore (int position) const;
     bool moveCaretWithTransaction (int newPos, bool selecting);
     void drawContent (Graphics&);
     void checkLayout();
-    void updateTextHolderSize (int, int);
-    void updateScrollbarVisibility (int, int);
-    float getWordWrapWidth() const;
-    float getMaximumWidth() const;
-    float getMaximumHeight() const;
+    int getWordWrapWidth() const;
+    int getMaximumTextWidth() const;
+    int getMaximumTextHeight() const;
     void timerCallbackInt();
     void checkFocus();
     void repaintText (Range<int>);
     void scrollByLines (int deltaLines);
     bool undoOrRedo (bool shouldUndo);
     UndoManager* getUndoManager() noexcept;
+    void setSelection (Range<int>) noexcept;
+    Point<int> getTextOffset() const noexcept;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (TextEditor)
 };
