@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -74,7 +74,7 @@ public:
 
             if (DragAndDropContainer* const dnd = DragAndDropContainer::findParentDragContainerFor (this))
             {
-                dnd->startDragging (Toolbar::toolbarDragDescriptor, getParentComponent(), Image(), true, nullptr, &e.source);
+                dnd->startDragging (Toolbar::toolbarDragDescriptor, getParentComponent(), ScaledImage(), true, nullptr, &e.source);
 
                 if (ToolbarItemComponent* const tc = getToolbarItemComponent())
                 {
@@ -239,4 +239,17 @@ void ToolbarItemComponent::setEditingMode (const ToolbarEditingMode newMode)
     }
 }
 
+//==============================================================================
+std::unique_ptr<AccessibilityHandler> ToolbarItemComponent::createAccessibilityHandler()
+{
+    const auto shouldItemBeAccessible = (itemId != ToolbarItemFactory::separatorBarId
+                                      && itemId != ToolbarItemFactory::spacerId
+                                      && itemId != ToolbarItemFactory::flexibleSpacerId);
+
+    if (! shouldItemBeAccessible)
+        return createIgnoredAccessibilityHandler (*this);
+
+    return std::make_unique<ButtonAccessibilityHandler> (*this, AccessibilityRole::button);
+}
+
 } // namespace juce
