@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -110,7 +110,7 @@ public:
     /** Finds the character at a given on-screen position.
         The coordinates are relative to this component's top-left origin.
     */
-    CodeDocument::Position getPositionAt (int x, int y);
+    CodeDocument::Position getPositionAt (int x, int y) const;
 
     /** Returns the start of the selection as a position. */
     CodeDocument::Position getSelectionStart() const            { return selectionStart; }
@@ -404,6 +404,8 @@ private:
     class GutterComponent;
     std::unique_ptr<GutterComponent> gutter;
 
+    class CodeEditorAccessibilityHandler;
+
     enum DragType
     {
         notDragging,
@@ -432,6 +434,7 @@ private:
     int getGutterSize() const noexcept;
 
     //==============================================================================
+    std::unique_ptr<AccessibilityHandler> createAccessibilityHandler() override;
     void insertText (const String&);
     virtual void updateCaretPosition();
     void updateScrollBars();
@@ -442,6 +445,7 @@ private:
     void indentSelectedLines (int spacesToAdd);
     bool skipBackwardsToPreviousTab();
     bool performCommand (CommandID);
+    void setSelection (CodeDocument::Position, CodeDocument::Position);
 
     int indexToColumn (int line, int index) const noexcept;
     int columnToIndex (int line, int column) const noexcept;
