@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -31,10 +31,6 @@ RecentlyOpenedFilesList::RecentlyOpenedFilesList()
 {
 }
 
-RecentlyOpenedFilesList::~RecentlyOpenedFilesList()
-{
-}
-
 //==============================================================================
 void RecentlyOpenedFilesList::setMaxNumberOfItems (const int newMaxNumber)
 {
@@ -157,7 +153,7 @@ void RecentlyOpenedFilesList::forgetRecentFileNatively (const File& file)
         // from the recent list, so we clear them all and add them back excluding
         // the specified file
 
-        NSDocumentController* sharedDocController = [NSDocumentController sharedDocumentController];
+        auto sharedDocController = [NSDocumentController sharedDocumentController];
         auto recentDocumentURLs  = [sharedDocController recentDocumentURLs];
 
         [sharedDocController clearRecentDocuments: nil];
