@@ -3,7 +3,7 @@
 
    This file is part of the Water library.
    Copyright (c) 2016 ROLI Ltd.
-   Copyright (C) 2017-2018 Filipe Coelho <falktx@falktx.com>
+   Copyright (C) 2017-2022 Filipe Coelho <falktx@falktx.com>
 
    Permission is granted to use this software under the terms of the ISC license
    http://www.isc.org/downloads/software-support-policy/isc-license/
@@ -28,6 +28,9 @@
 
 #include "../containers/Array.h"
 #include "../misc/Result.h"
+#include "../text/String.h"
+
+#include <vector>
 
 namespace water {
 
@@ -89,11 +92,6 @@ public:
     /** Copies from another file object. */
     File& operator= (const File& otherFile);
 
-   #if WATER_COMPILER_SUPPORTS_MOVE_SEMANTICS
-    File (File&&) noexcept;
-    File& operator= (File&&) noexcept;
-   #endif
-
     //==============================================================================
     /** Checks whether the file actually exists.
 
@@ -496,10 +494,10 @@ public:
 
         @see getNumberOfChildFiles, DirectoryIterator
     */
-    int findChildFiles (Array<File>& results,
-                        int whatToLookFor,
-                        bool searchRecursively,
-                        const String& wildCardPattern = "*") const;
+    uint findChildFiles (std::vector<File>& results,
+                         int whatToLookFor,
+                         bool searchRecursively,
+                         const String& wildCardPattern = "*") const;
 
     /** Searches inside a directory and counts how many files match a wildcard pattern.
 
@@ -517,8 +515,8 @@ public:
         @returns                the number of matches found
         @see findChildFiles, DirectoryIterator
     */
-    int getNumberOfChildFiles (int whatToLookFor,
-                               const String& wildCardPattern = "*") const;
+    uint getNumberOfChildFiles (int whatToLookFor,
+                                const String& wildCardPattern = "*") const;
 
     /** Returns true if this file is a directory that contains one or more subdirectories.
         @see isDirectory, findChildFiles
@@ -664,7 +662,10 @@ public:
         currentExecutableFile,
 
         /** In a plugin, this will return the path of the host executable. */
-        hostApplicationPath
+        hostApplicationPath,
+
+        /** Windows specific paths */
+        winAppData, winProgramFiles
     };
 
     /** Finds the location of a special type of file or directory, such as a home folder or
