@@ -1,6 +1,6 @@
 /*
  * Carla Native Plugins
- * Copyright (C) 2013-2021 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2013-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,7 +18,6 @@
 #ifndef AUDIO_BASE_HPP_INCLUDED
 #define AUDIO_BASE_HPP_INCLUDED
 
-#include "CarlaThread.hpp"
 #include "CarlaMathUtils.hpp"
 
 extern "C" {
@@ -39,12 +38,14 @@ extern "C" {
 # pragma GCC diagnostic pop
 #endif
 
-#ifdef CARLA_OS_WIN
+#if defined(CARLA_OS_WIN)
 # include <windows.h>
 # define CARLA_MLOCK(ptr, size) VirtualLock((ptr), (size))
-#else
+#elif !defined(CARLA_OS_WASM)
 # include <sys/mman.h>
 # define CARLA_MLOCK(ptr, size) mlock((ptr), (size))
+#else
+# define CARLA_MLOCK(ptr, size)
 #endif
 
 // #define DEBUG_FILE_OPS
@@ -220,7 +221,7 @@ struct AudioFilePool {
         return true;
     }
 
-    CARLA_DECLARE_NON_COPY_STRUCT(AudioFilePool)
+    CARLA_DECLARE_NON_COPYABLE(AudioFilePool)
 };
 
 class AudioFileReader
@@ -862,7 +863,7 @@ private:
 #endif
     }
 
-    CARLA_DECLARE_NON_COPY_STRUCT(AudioFileReader)
+    CARLA_DECLARE_NON_COPYABLE(AudioFileReader)
 };
 
 #endif // AUDIO_BASE_HPP_INCLUDED
